# orbipay/paymentsapi-client

orbipay/paymentsapi-client ---> PHP client for Orbipay Payments API.
The Payments API enables businesses to accept credit, debit, and bank account payments.

- API version: 1.4.0
- Package version: 1.4.0

## Requirements

PHP 5.5 and later

- "ext-curl": "*",
- "ext-json": "*"
- "ext-mbstring": "*",
- "guzzlehttp/guzzle": "^6.2"
- "lf4php/lf4php": "^4.2.2"


## Installation & Usage

### Manual Installation

1. Download the files
2. Add Following lines in your composer.json

```
     "repositories": [
         {
             "type": "path",
             "url": "/home/user/Downloads/orbipay-paymentsapi-client",
             "options": {
                 "symlink": false
             }
         }
     ]
```

For Example:

```json
 {
     "name": "example/client",
     "description": "example client",
     "require": {
         "php": ">=5.5",
         "ext-json": "*",
         "monolog/monolog": "1.23.*",
         "lf4php/lf4php-monolog": "^3.1.2",
         "orbipay/paymentsapi-client": "1.4.0"
     },
     "minimum-stability": "stable",
     "repositories": [
         {
             "type": "path",
             "url": "/home/user/Downloads/orbipay-paymentsapi-client",
             "options": {
                 "symlink": false
             }
         }
     ],
     "autoload": {
         "psr-4": {
             "MiniUpload\\": "src/"
         }
     } 
 }
```

```
composer install
```

###Logging
To see the logs Please [follow](https://packagist.org/packages/lf4php/lf4php).


###Timezone
You will need to specify your server's timezone before using this library. You can do this in your php.ini file:

date.timezone = 'America/Los_Angeles'
or in your PHP script:
date_default_timezone_set('America/Los_Angeles');

###Timeout
This library provides a way to configure the http_timeout through the environment variable 'ORBIPAY_API_CLIENT_TIMEOUT_SECONDS'. For example in linux,
```shell
export ORBIPAY_API_CLIENT_TIMEOUT_SECONDS=60
```

## Getting Started

Please follow the [installation procedure](#installation--usage) and then run the following:

```php
<?php
require_once(__DIR__ . './vendor/autoload.php');

$channel = "channel_example"; // string | The channel through which the API is invoked. Please contact <a href = \"mailto: support@billerpayments.com\">support@billerpayments.com</a> to configure channels.
$client_key = "client_key_example"; // string | The unique identifier assigned by Orbipay to the client.
$idempotent_request_key = "idempotent_request_key_example"; // string | The unique token sent by the client to identify the request. This is used to identify retrials and duplicate requests.
$requestor = "requestor_example"; // string | The identifier for the requestor of the API, in Orbipay. The value to be passed in the requestor field would depend on the requestor type. For customer, it would be the Orbipay generated customer id, for user, it would be the Orbipay user login id and for external user, it would be the client's external agent id. In the case of self registration, where there is no external identifier, the requestor can be set to 'guest'.
$requestor_type = "requestor_type_example"; // string | Type of the requestor of the API. Please note that requestor types 'customer' and 'external_user' are supported only by the Payments API. The requestor type 'payee' is supported only by the Payouts API.
$create_customer_request = new \Orbipay\PaymentsApi\Client\Swagger\Model\CreateCustomerRequest(); // \Orbipay\PaymentsApi\Client\Swagger\Model\CreateCustomerRequest | The JSON that contains all the attributes of customer to be created.
$ID_CUSTOMER= "ID_CUSTOMER_example"; // string | The unique identifier assigned by Orbipay to the customer.
$secret = 'secret_example'; // string | The secret key specific to the client that is shared with the client during on boarding.

$attributes = new \Orbipay\PaymentsApi\Client\Model\Base\Attributes();
$attributes->add('_test_header', 'testHeaderValue');
$invocation_context = new \Orbipay\PaymentsApi\Client\Model\Base\InvocationContext($idempotent_request_key, $secret, $attributes);

$customerRequest = new \Orbipay\PaymentsApi\Client\Model\Customer\Customer('12345897');

try {
$response = $customerRequest
    ->forClient($client_key)
    ->createdBy($requestor, \Orbipay\PaymentsApi\Client\Enum\RequestorType::fromValue($requestor))
    ->videChannel($channel)
    ->get($invocation_context); // for live mode
    //->get($invocation_context, \Orbipay\PaymentsApi\Client\Enum\InvocationMode::SAND_BOX()); //for sandbox mode
    //->get($invocation_context, \Orbipay\PaymentsApi\Client\Enum\InvocationMode::TEST(), 'http://localhost:8080/payments/v1'); // for test mode
    
} catch (\Orbipay\PaymentsApi\Client\Exception\OrbipayApiException $exception) {
    echo 'exception_message: ' . $exception->getMessage();
    $errors = $exception->getErrors();
    foreach ($errors as $error){
        echo 'code: ' . $error->getCode();
        echo 'error_message: ' . $error->getMessage();
        echo 'field: ' . $error->getField();
    }
} catch(\Exception $e){
    echo 'exception_message: ' . $e->getMessage();
}

?>
```



## Author
support@billerpayments.com

