<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\CustomerAccount;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\CustomerAccount\CustomerAccount;
use Orbipay\PaymentsApi\Client\Swagger\Model\RenewCustomerAccountRequest;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\CustomerAccountApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;

class RenewCustomerAccountHandler
{
    /**
     * @param CustomerAccount $customerAccount
     * @return CustomerAccount
     * @throws OrbipayApiException
     */
    public function process(CustomerAccount $customerAccount)
    {
        return $this->callAPI($customerAccount);
    }

    /**
     * @param CustomerAccount $customerAccount
     * @return CustomerAccount
     * @throws OrbipayApiException
     */
    private function callAPI(CustomerAccount $customerAccount)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($customerAccount->getApiEndPoint());
        $configuration->setClientKey($customerAccount->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($customerAccount->getInvocationContext()->getSecret());

        $customerAccountApi = new CustomerAccountApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\CustomerAccountResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $customerAccountApi->renewCustomerAccount($customerAccount->getInvocationContext()->getChannel(),
                $customerAccount->getInvocationContext()->getClientKey(),
                $customerAccount->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $customerAccount->getInvocationContext()->getIdempotentRequestKey(),
                $customerAccount->getInvocationContext()->getRequestor(),
                $customerAccount->getInvocationContext()->getRequestorType()->value(),
                $customerAccount->getPathParamData()->getIdCustomerAccount(),
                $customerAccount->getPathParamData()->getIdCustomer(),
                $this->getBody($customerAccount),
                RequestWrapper::wrapXOPAYHeaders($customerAccount->getInvocationContext()->getAttributes()));
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $customerAccount, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $customerAccount);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $customerAccount, $e->getCode(), $e);
        }

        return $this->wrapResponse($response);
    }

    private function getBody(CustomerAccount $customerAccount = null)
    {
        $customerAccountRequest = new RenewCustomerAccountRequest();
        $customerAccountRequest->setAccountHolderName($customerAccount->getAccountHolderName());
        $customerAccountRequest->setNickname($customerAccount->getNickname());
        $customerAccountRequest->setAddress(RequestWrapper::wrapAddress($customerAccount->getAddress()));
        $customerAccountRequest->setCustomerAccountReference($customerAccount->getCustomerAccountReference());
        $customerAccountRequest->setAccountNumber($customerAccount->getAccountNumber());
        $customerAccountRequest->setCurrentBalance($customerAccount->getCurrentBalance());
        $customerAccountRequest->setCurrentStatementBalance($customerAccount->getCurrentStatementBalance());
        $customerAccountRequest->setMinimumPaymentDue($customerAccount->getMinimumPaymentDue());
        $customerAccountRequest->setPastAmountDue($customerAccount->getPastAmountDue());
        $customerAccountRequest->setPaymentDueDate($customerAccount->getPaymentDueDate());
        $customerAccountRequest->setStatementDate($customerAccount->getStatementDate());
        $customerAccountRequest->setCustomFields((object)$customerAccount->getCustomFields());
        $customerAccountRequest->setComments($customerAccount->getComments());
        return $customerAccountRequest;
    }

    /**
     * @param array $apiResponse
     * @return CustomerAccount
     */
    private function wrapResponse(array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($customerAccountResponse, $statusCode, $headers) = $apiResponse;

        $customerAccount = ResponseWrapper::wrapCustomerAccount($customerAccountResponse);
        $customerAccount->setInvocationContext(ResponseWrapper::getInvocationContext($headers));
        return $customerAccount;
    }
}