<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\FeePayment;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\AppErrorCodes;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\DataType;
use Orbipay\PaymentsApi\Client\Enum\Mandatory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Model\FeePayment\FeePayment;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\FeePaymentApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;

class GetFeePaymentHandler
{
    /**
     * @param FeePayment $feePayment
     * @return FeePayment
     * @throws OrbipayApiException
     */
    public function process(FeePayment $feePayment)
    {
        $this->validate($feePayment);
        return $this->callAPI($feePayment);
    }

    /**
     * @param FeePayment $feePayment
     * @return FeePayment
     * @throws OrbipayApiException
     */
    private function callAPI(FeePayment $feePayment)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($feePayment->getApiEndPoint());
        $configuration->setClientKey($feePayment->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($feePayment->getInvocationContext()->getSecret());

        $feePaymentApi = new FeePaymentApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\PaymentResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $feePaymentApi->getFeePayment($feePayment->getInvocationContext()->getChannel(),
                $feePayment->getInvocationContext()->getClientKey(),
                $feePayment->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $feePayment->getInvocationContext()->getIdempotentRequestKey(),
                $feePayment->getInvocationContext()->getRequestor(),
                $feePayment->getInvocationContext()->getRequestorType()->value(),
                $feePayment->getPathParamData()->getIdFee(),
                $feePayment->getPathParamData()->getIdCustomer(),
                RequestWrapper::wrapXOPAYHeaders($feePayment->getInvocationContext()->getAttributes()));
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $feePayment, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $feePayment);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $feePayment, $e->getCode(), $e);
        }

        return $this->wrapResponse($response);
    }

    /**
     * @param FeePayment $feePayment
     * @throws OrbipayApiException
     */
    private function validate(FeePayment $feePayment = null)
    {
        $exception = new OrbipayApiException(RestConstants::CLIENT_SIDE_VALIDATION_FAILURE, $feePayment);

        ValidationUtil::validate($feePayment->getPathParamData()->getIdCustomer(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_CUSTOMER, AppErrorCodes::ID_CUSTOMER_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));
        ValidationUtil::validate($feePayment->getPathParamData()->getIdFee(), DataType::STRING, Mandatory::YES, $exception, HttpParamConstants::PATH_PARAM_ID_FEE, AppErrorCodes::ID_FEE_INVALID, CommonUtil::getValidationMethod(ValidationUtil::class, 'isValidId'));

        ValidationUtil::validateInvocationContext($feePayment->getInvocationContext(), $exception);

        if (count($exception->getErrors()) > 0) {
            throw $exception;
        }
    }

    /**
     * @param array $apiResponse
     * @return FeePayment
     */
    private function wrapResponse(array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($feePaymentResponse, $statusCode, $headers) = $apiResponse;

        $feePayment = ResponseWrapper::wrapFeePayment($feePaymentResponse);
        $feePayment->setInvocationContext(ResponseWrapper::getInvocationContext($headers));
        return $feePayment;
    }
}