<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 5:21 PM
 */

namespace Orbipay\PaymentsApi\Client\Handler\Payment;


use GuzzleHttp\Exception\GuzzleException;
use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Exception\OrbipayApiException;
use Orbipay\PaymentsApi\Client\Swagger\Model\SimplePaymentRequest;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\RequestWrapper;
use Orbipay\PaymentsApi\Client\Util\ResponseWrapper;
use Orbipay\PaymentsApi\Client\Util\TimeUtil;
use Orbipay\PaymentsApi\Client\Swagger\Api\PaymentApi;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Configuration;

class CreateSimplePaymentHandler
{
    /**
     * @param SimplePayment $payment
     * @return SimplePayment
     * @throws OrbipayApiException
     */
    public function process(SimplePayment $payment)
    {
        return $this->callAPI($payment);
    }

    /**
     * @param SimplePayment $payment
     * @return SimplePayment
     * @throws OrbipayApiException
     */
    private function callAPI(SimplePayment $payment)
    {
        $logger = LoggerFactory::getLogger(CommonUtil::class);
        $configuration = new Configuration();
        $configuration->setHost($payment->getApiEndPoint());
        $configuration->setClientKey($payment->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($payment->getInvocationContext()->getSecret());

        $paymentApi = new PaymentApi(null, $configuration, null);

        /**
         * @var array of \Orbipay\PaymentsApi\Client\Swagger\Model\SimplePaymentResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;

        try {
            $response = $paymentApi->simplePayment($payment->getInvocationContext()->getChannel(),
                $payment->getInvocationContext()->getClientKey(),
                $payment->getInvocationContext()->getProduct()->value(),
                TimeUtil::getTimeStamp(),
                $payment->getInvocationContext()->getIdempotentRequestKey(),
                $payment->getInvocationContext()->getRequestor(),
                $payment->getInvocationContext()->getRequestorType()->value(),
                $this->getBody($payment),
                RequestWrapper::wrapXOPAYHeaders($payment->getInvocationContext()->getAttributes()),
                $payment->getQueryParamData()->getSaveFundingAccount());
        } catch (GuzzleException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $payment, $e->getCode(), $e);
        } catch (ApiException $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::constructFromHttpResponse($e, $payment);
        } catch (\Exception $e) {
            $logger->error($e->getMessage(), [], $e);
            throw OrbipayApiException::getDefaultException($e->getMessage(), $payment, $e->getCode(), $e);
        }

        return $this->wrapResponse($response);
    }

    /**
     * @param array $apiResponse
     * @return SimplePayment
     */
    private function wrapResponse(array $apiResponse = null)
    {
        if ($apiResponse === null)
            return null;

        list($paymentResponse, $statusCode, $headers) = $apiResponse;

        $payment = ResponseWrapper::wrapSimplePayment($paymentResponse);
        $payment->setInvocationContext(ResponseWrapper::getInvocationContext($headers));
        return $payment;
    }

    private function getBody(SimplePayment $payment)
    {
        $body = new SimplePaymentRequest();
        $body->setAmount($payment->getAmount());
        $body->setCardCvvNumber($payment->getCardCvvNumber());
        $body->setPaymentDate($payment->getPaymentDate());
        $body->setPaymentRequestDate($payment->getPaymentRequestDate());
        $body->setPaymentAmountType($payment->getPaymentAmountType() !== null ? $payment->getPaymentAmountType()->value() : null);
        $body->setFee(RequestWrapper::getFeeRequest($payment->getFee()));
        $body->setCustomer(RequestWrapper::getSimplePaymentCustomerRequest($payment->getCustomer()));
        $body->setCustomerAccount(RequestWrapper::getSimplePaymentCustomerAccountRequest($payment->getCustomerAccount()));
        $body->setFundingAccount(RequestWrapper::getSimplePaymentFundingAccountRequest($payment->getFundingAccount()));
        $body->setPaymentReference($payment->getPaymentReference());
        $body->setComments($payment->getComments());
        $body->setCustomFields($payment->getCustomFields());
        return $body;
    }
}