<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/18/18
 * Time: 8:15 PM
 */

namespace Orbipay\PaymentsApi\Client\Model\Base;


use JsonSerializable;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\InvocationMode;
use Orbipay\PaymentsApi\Client\Enum\Product;
use Orbipay\PaymentsApi\Client\Enum\RequestorType;


use Orbipay\PaymentsApi\Client\Util\CommonUtil;

class Base implements JsonSerializable
{
    const MODE_LIVE = 1;
    const MODE_SAND_BOX = 2;
    const MODE_TEST = 3;

    /**
     * @var InvocationMode
     */
    private $invocation_mode;

    /**
     * @var InvocationContext
     */
    private $invocation_context;

    /**
     * @var string
     */
    protected $api_end_point;

    /**
     * @var PathParamData
     */
    private $path_param_data;

    /**
     * @var FormParamData
     */
    private $form_param_data;

    /**
     * @var QueryParamData
     */
    private $query_param_data;

    /**
     * @internal
     * @return InvocationMode
     */
    public function getInvocationMode()
    {
        return $this->invocation_mode;
    }

    /**
     * @internal
     * @param InvocationMode $invocation_mode
     */
    public function setInvocationMode($invocation_mode)
    {
        $this->invocation_mode = $invocation_mode;
    }

    /**
     * @return InvocationContext
     */
    public function getInvocationContext()
    {
        return $this->invocation_context;
    }

    /**
     * @param InvocationContext $invocation_context
     */
    public function setInvocationContext(InvocationContext $invocation_context = null)
    {
        $this->invocation_context = $invocation_context;
    }

    /**
     * @return string
     */
    public function getApiEndPoint()
    {
        return $this->api_end_point;
    }

    /**
     * @param string $channel
     */
    protected function videChannel($channel = null)
    {
        $invocationContext = $this->getInvocationContext();
        if (!isset($invocationContext)) {
            $invocationContext = new InvocationContext();
        }
        $invocationContext->setChannel($channel);
        $this->setInvocationContext($invocationContext);
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestorType
     */
    protected function requestedBy($requestor = null, RequestorType $requestorType = null)
    {
        $invocationContext = $this->getInvocationContext();
        if (!isset($invocationContext)) {
            $invocationContext = new InvocationContext();
        }
        $invocationContext->setRequestor($requestor);
        $invocationContext->setRequestorType($requestorType);
        $this->setInvocationContext($invocationContext);
    }

    /**
     * @param string $client_key
     */
    protected function forClient($client_key = null)
    {
        $invocationContext = $this->getInvocationContext();
        if (!isset($invocationContext)) {
            $invocationContext = new InvocationContext();
        }
        $invocationContext->setClientKey($client_key);
        $this->setInvocationContext($invocationContext);
    }


    /**
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     */
    protected function setAPIEndpoint(InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->invocation_mode = $invocationMode;

        if (!isset($invocationMode)) {
            $this->api_end_point = RestConstants::PRODUCTION_API_END_POINT;
        } else {
            if ($invocationMode->value() === InvocationMode::TEST) {
                $this->api_end_point = $apiEndPoint;
            } elseif ($invocationMode->value() === InvocationMode::SAND_BOX) {
                $this->api_end_point = RestConstants::SANDBOX_API_END_POINT;
            } else {
                $this->api_end_point = RestConstants::PRODUCTION_API_END_POINT;
            }
        }
    }

    /**
     * @param InvocationContext $request
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     */
    public function withContext(InvocationContext $request, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->setAPIEndpoint($invocationMode, $apiEndPoint);

        if (!isset($this->invocation_context)) {
            $this->invocation_context = new InvocationContext();
        }

        if (isset($request)) {
            $this->invocation_context->setAttributes($request->getAttributes());
            $this->invocation_context->setIdempotentRequestKey($request->getIdempotentRequestKey());
            $this->invocation_context->setProduct(Product::ORBIPAY_PAYMENTS());
            $this->invocation_context->setSecret($request->getSecret());
        }
    }

    /**
     * @return PathParamData
     */
    public function getPathParamData()
    {
        return $this->path_param_data;
    }

    /**
     * @param PathParamData $path_param_data
     */
    public function setPathParamData(PathParamData $path_param_data = null)
    {
        $this->path_param_data = $path_param_data;
    }

    /**
     * @return FormParamData
     */
    public function getFormParamData()
    {
        return $this->form_param_data;
    }

    /**
     * @param FormParamData $form_param_data
     */
    public function setFormParamData(FormParamData $form_param_data = null)
    {
        $this->form_param_data = $form_param_data;
    }

    /**
     * @return QueryParamData
     */
    public function getQueryParamData()
    {
        return $this->query_param_data;
    }

    /**
     * @param QueryParamData $query_param_data
     */
    public function setQueryParamData(QueryParamData $query_param_data = null)
    {
        $this->query_param_data = $query_param_data;
    }

    /**
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        return CommonUtil::json_serialize(get_object_vars($this));
    }
}