<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/19/18
 * Time: 3:37 PM
 */

namespace Orbipay\PaymentsApi\Client\Model\Base;


use Orbipay\PaymentsApi\Client\Enum\Product;
use Orbipay\PaymentsApi\Client\Enum\RequestorType;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;

class InvocationContext implements \JsonSerializable
{
    /**
     * @var string
     */
    private $channel;

    /**
     * @var string
     */
    private $clientKey;

    /**
     * @var Product
     */
    private $product;

    /**
     * @var string
     */
    private $timestamp;

    /**
     * @var string
     */
    private $idempotentRequestKey;

    /**
     * @var string
     */
    private $requestor;

    /**
     * @var RequestorType
     */
    private $requestorType;

    /**
     * @var Attributes
     */
    private $attributes;

    /**
     * @var string
     */
    private $requestUuid;

    /**
     * @var string
     */
    private $responseCodes;

    /**
     * @var string
     */
    private $responseText;

    /**
     * @var string
     */
    private $secret;

    /**
     * @internal
     * @param string $idempotentRequestKey
     */
    public function setIdempotentRequestKey($idempotentRequestKey = null)
    {
        $this->idempotentRequestKey = $idempotentRequestKey;
    }

    /**
     * InvocationContext constructor.
     * @param string $idempotentRequestKey
     * @param string $clientSignatureKey
     * @param Attributes $attributes
     */
    public function __construct($idempotentRequestKey = null, $clientSignatureKey = null, Attributes $attributes = null)
    {
        $this->idempotentRequestKey = $idempotentRequestKey;
        $this->secret = $clientSignatureKey;
        $this->attributes = $attributes;
    }

    /**
     * @return string
     */
    public function getChannel()
    {
        return $this->channel;
    }

    /**
     * @internal
     * @param string $channel
     */
    public function setChannel($channel = null)
    {
        $this->channel = $channel;
    }

    /**
     * @return string
     */
    public function getClientKey()
    {
        return $this->clientKey;
    }

    /**
     * @internal
     * @param string $clientKey
     */
    public function setClientKey($clientKey = null)
    {
        $this->clientKey = $clientKey;
    }

    /**
     * @return Product
     */
    public function getProduct()
    {
        return $this->product;
    }


    /**
     * @internal
     * @param Product $product
     */
    public function setProduct(Product $product = null)
    {
        $this->product = $product;
    }

    /**
     * @return string
     */
    public function getTimestamp()
    {
        return $this->timestamp;
    }

    /**
     * @internal
     * @param string $timestamp
     */
    public function setTimestamp($timestamp = null)
    {
        $this->timestamp = $timestamp;
    }

    /**
     * @return string
     */
    public function getIdempotentRequestKey()
    {
        return $this->idempotentRequestKey;
    }

    /**
     * @return string
     */
    public function getRequestor()
    {
        return $this->requestor;
    }

    /**
     * @internal
     * @param string $requestor
     */
    public function setRequestor($requestor = null)
    {
        $this->requestor = $requestor;
    }

    /**
     * @return RequestorType
     */
    public function getRequestorType()
    {
        return $this->requestorType;
    }


    /**
     * @internal
     * @param RequestorType $requestorType
     */
    public function setRequestorType(RequestorType $requestorType = null)
    {
        $this->requestorType = $requestorType;
    }

    /**
     * @return Attributes
     */
    public function getAttributes()
    {
        return $this->attributes;
    }

    /**
     * @return string
     */
    public function getRequestUuid()
    {
        return $this->requestUuid;
    }

    /**
     * @internal
     * @param string $requestUuid
     */
    public function setRequestUuid($requestUuid = null)
    {
        $this->requestUuid = $requestUuid;
    }

    /**
     * @return string
     */
    public function getResponseCodes()
    {
        return $this->responseCodes;
    }

    /**
     * @internal
     * @param string $responseCodes
     */
    public function setResponseCodes($responseCodes = null)
    {
        $this->responseCodes = $responseCodes;
    }

    /**
     * @return string
     */
    public function getResponseText()
    {
        return $this->responseText;
    }

    /**
     * @internal
     * @param string $responseText
     */
    public function setResponseText($responseText = null)
    {
        $this->responseText = $responseText;
    }

    /**
     * @return string
     */
    public function getSecret()
    {
        return $this->secret;
    }

    /**
     * @param string $secret
     */
    public function setSecret($secret = null)
    {
        $this->secret = $secret;
    }

    /**
     * @internal
     * @param Attributes $attributes
     */
    public function setAttributes(Attributes $attributes = null)
    {
        $this->attributes = $attributes;
    }

    /**
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        return CommonUtil::json_serialize(get_object_vars($this));
    }
}