<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/19/18
 * Time: 4:55 PM
 */

namespace Orbipay\PaymentsApi\Client\Model\Base;


class MultiMap implements \JsonSerializable
{
    /**
     * @var array
     */
    private $map = array();

    /**
     * @var integer
     */
    private $size = 0;

    /**
     * clears the multi map.
     */
    public function clear()
    {
        unset($this->map);
        $this->map = array();
        $this->size = 0;
    }

    /**
     * @param string $key
     * @param string $value
     * @return boolean
     */
    public function containsEntry($key, $value)
    {
        if (!$this->containsKey($key)) return false;
        $values = $this->map[$key];
        foreach ($values as $k => $v) {
            if ($value == $v) return true;
        }
        return false;
    }

    /**
     * @param string $key
     * @return boolean
     */
    public function containsKey($key)
    {
        return array_key_exists($key, $this->map);
    }

    /**
     * @param string $value
     * @return boolean
     */
    public function containsValue($value)
    {
        foreach ($this->map as $k => $v) {
            if (in_array($value, $v)) return true;
        }
        return false;
    }

    /**
     * @param string $key
     * @return array
     */
    public function get($key)
    {
        if (!array_key_exists($key, $this->map)) return null;
        return $this->map[$key];
    }

    /**
     * @return boolean
     */
    public function isEmpty()
    {
        return ($this->size <= 0);
    }

    /**
     * @return array
     */
    public function keys()
    {
        $keys = array();
        foreach ($this->map as $k => $v) {
            $keys[] = $k;
        }
        return $keys;
    }

    /**
     * @return array
     */
    public function keySet()
    {
        return array_unique($this->keys());
    }

    /**
     * @param string $key
     * @param string $value
     * @return boolean
     */
    public function put($key, $value)
    {
        if (!array_key_exists($key, $this->map)) {
            $this->map[$key] = array($value);
        } else {
            $this->map[$key][] = $value;
        }
        $this->size++;
        return true;
    }

    /**
     * @param string $key
     * @param array $values
     * @return boolean
     */
    public function putAll($key, $values)
    {
        $result = false;
        foreach ($values as $v) {
            $result = $this->put($key, $v);
        }
        return $result;
    }

    /**
     * @param string $key
     * @param string $value
     * @return boolean
     */
    public function remove($key, $value)
    {
        if (!array_key_exists($key, $this->map)) return false;
        $result = false;
        $values = $this->map[$key];
        foreach ($values as $k => $v) {
            if ($value == $v) {
                unset($values[$k]);
                $result = true;
                $this->size--;
            }
        }
        $this->map[$key] = $values;
        return $result;
    }

    /**
     * @param string $key
     * @return boolean
     */
    public function removeAll($key)
    {
        if (!array_key_exists($key, $this->map)) return false;
        $values = $this->map[$key];
        $count = count($values);
        unset($this->map[$key]);
        $this->size = $this->size - $count;
        return true;
    }

    /**
     * @param string $key
     * @param array $values
     * @return boolean
     */
    public function replaceValues($key, array $values)
    {
        if (!array_key_exists($key, $this->map)) return false;
        $this->map[$key] = $values;
        return true;
    }

    /**
     * @return integer
     */
    public function size()
    {
        return $this->size;
    }

    /**
     * @return array
     */
    public function values()
    {
        $values = array();
        foreach ($this->map as $k => $v) {
            foreach ($v as $value) {
                $values[] = $value;
            }
        }
        return $values;
    }

    /**
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        return $this->map;
    }
}