<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/20/18
 * Time: 2:37 PM
 */

namespace Orbipay\PaymentsApi\Client\Model\Customer;


use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\CustomerStatus;
use Orbipay\PaymentsApi\Client\Enum\Gender;
use Orbipay\PaymentsApi\Client\Enum\InvocationMode;
use Orbipay\PaymentsApi\Client\Enum\RequestorType;
use Orbipay\PaymentsApi\Client\Handler\Customer\CreatorCustomerHandler;
use Orbipay\PaymentsApi\Client\Handler\Customer\GetCustomerHandler;
use Orbipay\PaymentsApi\Client\Handler\Customer\UpdateCustomerHandler;
use Orbipay\PaymentsApi\Client\Model\Base\Address;
use Orbipay\PaymentsApi\Client\Model\Base\AuditInfo;
use Orbipay\PaymentsApi\Client\Model\Base\Base;
use Orbipay\PaymentsApi\Client\Model\Base\InvocationContext;
use Orbipay\PaymentsApi\Client\Model\Base\PathParamData;
use Orbipay\PaymentsApi\Client\Model\CustomerAccount\CustomerAccounts;
use Orbipay\PaymentsApi\Client\Model\FundingAccount\FundingAccounts;
use Orbipay\PaymentsApi\Client\Model\Payment\Payments;
use Orbipay\PaymentsApi\Client\Model\PaymentSetup\PaymentSetups;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;

class Customer extends Base
{
    /**
     * @var string
     */
    private $customer_reference;

    /**
     * @var string
     */
    private $first_name;

    /**
     * @var string
     */
    private $last_name;

    /**
     * @var string
     */
    private $middle_name;

    /**
     * @var string
     */
    private $date_of_birth;

    /**
     * @var string
     */
    private $ssn;

    /**
     * @var string
     */
    private $locale;

    /**
     * @var string
     */
    private $email;

    /**
     * @var string
     */
    private  $registered_email;

    /**
     * @var string
     */
    private $home_phone;

    /**
     * @var string
     */
    private $work_phone;

    /**
     * @var string
     */
    private $mobile_phone;

    /**
     * @var Address
     */
    private $address;

    /**
     * @var CustomerAccounts
     */
    private $customer_accounts;

    /**
     * @var FundingAccounts
     */
    private $funding_accounts;

    /**
     * @var Payments
     */
    private $payments;

    /**
     * @var PaymentSetups
     */
    private $payment_setups;

    /**
     * @var Customers
     */
    private $customers;

    /**
     * @var string
     */
    private $id;

    /**
     * @var CustomerStatus
     */
    private $status;

    /**
     * @var string
     */
    private $url;

    /**
     * @var string
     */
    private $comments;

    /**
     * @var object
     */
    private $custom_fields;

    /**
     * @var Gender
     */
    private $gender;

    /**
     * @var AuditInfo
     */
    private $auditInfo;


    /**
     * Customer constructor.
     * @param string $ID_CUSTOMER
     */
    public function __construct($ID_CUSTOMER = null)
    {
        if ($this->getPathParamData() === null) {
            $this->setPathParamData(new PathParamData());
        }
        $this->getPathParamData()->setIdCustomer($ID_CUSTOMER);
        $this->id = $ID_CUSTOMER;
    }


    /**
     *
     * @param string $first_name
     * @param string $last_name
     * @param string $middle_name
     * @param string $date_of_birth
     * @param string $ssn
     * @param Gender $gender
     * @return $this
     */
    public function withDetails($first_name = null, $last_name = null, $middle_name = null, $date_of_birth = null, $ssn = null, Gender $gender = null, $registered_email = null)
    {
        $this->first_name = $first_name;
        $this->last_name = $last_name;
        $this->middle_name = $middle_name;
        $this->date_of_birth = $date_of_birth;
        $this->ssn = $ssn;
        $this->gender = $gender;
        $this->registered_email = $registered_email;
        return $this;
    }


    /**
     * @param string $customer_reference
     * @return $this
     */
    public function withReference($customer_reference = null)
    {
        $this->customer_reference = $customer_reference;
        return $this;
    }

    /**
     * @param CustomerStatus $status
     * @return $this
     */
    public function withStatus(CustomerStatus $status = null)
    {
        $this->status = $status;
        return $this;
    }

    /**
     * @param string $email
     * @param string $home_phone
     * @param string $work_phone
     * @param string $mobile_phone
     * @return $this
     */
    public function withContactInfo($email = null, $home_phone = null, $work_phone = null, $mobile_phone = null)
    {
        $this->email = $email;
        $this->home_phone = $home_phone;
        $this->work_phone = $work_phone;
        $this->mobile_phone = $mobile_phone;
        return $this;
    }

    /**
     * @param string $language
     * @param string $country
     * @return $this
     */
    public function withLocale($language = null, $country = null)
    {
        $this->locale = $language . RestConstants::LOCALE_DELIMITER . $country;
        return $this;
    }

    /**
     * @param CustomerAccounts $customer_accounts
     * @return $this
     */
    public function withAccounts(CustomerAccounts $customer_accounts = null)
    {
        $this->customer_accounts = $customer_accounts;
        return $this;
    }


    /**
     * @param Address $address
     * @return $this
     */
    public function withAddress(Address $address = null)
    {
        $this->address = $address;
        return $this;
    }

    /**
     * @param object $custom_fields
     * @return $this
     */
    public function withCustomFields($custom_fields = null)
    {
        if(ValidationUtil::isAssociativeArray($custom_fields)){
            $custom_fields = (object) $custom_fields;
        }
        $this->custom_fields = $custom_fields;
        return $this;
    }

    /**
     * @param string $comments
     * @return $this
     */
    public function withMemo($comments = null)
    {
        $this->comments = $comments;
        return $this;
    }

    /**
     * @internal
     * @param FundingAccounts $funding_accounts
     */
    public function setFundingAccounts(FundingAccounts $funding_accounts = null)
    {
        $this->funding_accounts = $funding_accounts;
    }

    /**
     * @internal
     * @param Payments $payments
     */
    public function setPayments(Payments $payments = null)
    {
        $this->payments = $payments;
    }

    /**
     * @internal
     * @param PaymentSetups $payment_setups
     */
    public function setPaymentSetups(PaymentSetups $payment_setups = null)
    {
        $this->payment_setups = $payment_setups;
    }

    /**
     * @internal
     * @param Customers $customers
     */

    public function setCustomers(Customers $customers = null)
    {
        $this->customers = $customers;
    }

    /**
     * @param string $url
     */
    public function setUrl($url = null)
    {
        $this->url = $url;
    }

    /**
     * @return string
     */
    public function getCustomerReference()
    {
        return $this->customer_reference;
    }

    /**
     * @return string
     */
    public function getFirstName()
    {
        return $this->first_name;
    }

    /**
     * @return string
     */
    public function getLastName()
    {
        return $this->last_name;
    }

    /**
     * @return string
     */
    public function getMiddleName()
    {
        return $this->middle_name;
    }

    /**
     * @return string
     */
    public function getDateOfbirth()
    {
        return $this->date_of_birth;
    }

    /**
     * @return string
     */
    public function getSsn()
    {
        return $this->ssn;
    }

    /**
     * @return string
     */
    public function getLocale()
    {
        return $this->locale;
    }

    /**
     * @return string
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * @return string
     */
    public function getRegisteredEmail()
    {
        return $this->registered_email;
    }

    /**
     * @return string
     */
    public function getHomePhone()
    {
        return $this->home_phone;
    }

    /**
     * @return string
     */
    public function getWorkPhone()
    {
        return $this->work_phone;
    }

    /**
     * @return string
     */
    public function getMobilePhone()
    {
        return $this->mobile_phone;
    }

    /**
     * @return Address
     */
    public function getAddress()
    {
        return $this->address;
    }

    /**
     * @return CustomerAccounts
     */
    public function getCustomerAccounts()
    {
        return $this->customer_accounts;
    }

    /**
     * @return FundingAccounts
     */
    public function getFundingAccounts()
    {
        return $this->funding_accounts;
    }

    /**
     * @return Payments
     */
    public function getPayments()
    {
        return $this->payments;
    }

    /**
     * @return PaymentSetups
     */
    public function getPaymentSetups()
    {
        return $this->payment_setups;
    }

    /**
     * @internal
     * @return Customers
     */
    public function getCustomers()
    {
        return $this->customers;
    }

    /**
     * @return string
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return CustomerStatus
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * @return string
     */
    public function getComments()
    {
        return $this->comments;
    }

    /**
     * @return object
     */
    public function getCustomFields()
    {
        return $this->custom_fields;
    }

    /**
     * @return Gender
     */
    public function getGender()
    {
        return $this->gender;
    }

    /**
     * @return AuditInfo
     */
    public function getAuditInfo()
    {
        return $this->auditInfo;
    }

    /**
     * @param AuditInfo $auditInfo
     */
    public function setAuditInfo(AuditInfo $auditInfo = null)
    {
        $this->auditInfo = $auditInfo;
    }



    /**
     * @param string $client_key
     * @return $this
     */
    public function forClient($client_key = null)
    {
        parent::forClient($client_key);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function createdBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function updatedBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function retrievedBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $channel
     * @return $this
     */
    public function videChannel($channel = null)
    {
        parent::videChannel($channel);
        return $this;
    }


    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return Customer
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function create(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new CreatorCustomerHandler();
        return $handler->process($this);
    }

    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return Customer
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function update(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new UpdateCustomerHandler();
        return $handler->process($this);
    }

    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return Customer
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function get(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new GetCustomerHandler();
        return $handler->process($this);
    }

    /**
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        return CommonUtil::json_serialize(get_object_vars($this), parent::jsonSerialize());
    }
}