<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/21/18
 * Time: 5:31 PM
 */

namespace Orbipay\PaymentsApi\Client\Model\CustomerAccount;


use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Enum\CustomerAccountStatus;
use Orbipay\PaymentsApi\Client\Enum\InvocationMode;
use Orbipay\PaymentsApi\Client\Enum\RequestorType;
use Orbipay\PaymentsApi\Client\Handler\CustomerAccount\GetCustomerAccountHandler;
use Orbipay\PaymentsApi\Client\Handler\CustomerAccount\RenewCustomerAccountHandler;
use Orbipay\PaymentsApi\Client\Model\Base\Address;
use Orbipay\PaymentsApi\Client\Model\Base\AuditInfo;
use Orbipay\PaymentsApi\Client\Model\Base\Base;
use Orbipay\PaymentsApi\Client\Model\Base\InvocationContext;
use Orbipay\PaymentsApi\Client\Model\Base\PathParamData;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;

class CustomerAccount extends Base
{
    /**
     * @var string
     */
    private $account_holder_name;

    /**
     * @var string
     */
    private $nickname;

    /**
     * @var string
     */
    private $account_number;

    /**
     * @var CustomerAccountStatus
     */
    private $status;

    /**
     * @var string
     */
    private $customer_account_reference;

    /**
     * @var string
     */
    private $current_balance;

    /**
     * @var string
     */
    private $current_statement_balance;

    /**
     * @var string
     */
    private $minimum_payment_due;

    /**
     * @var string
     */
    private $past_amount_due;

    /**
     * @var string
     */
    private $payment_due_date;

    /**
     * @var string
     */
    private $statement_date;

    /**
     * @var Address
     */
    private $address;

    /**
     * @var object
     */
    private $custom_fields;

    /**
     * @var string
     */
    private $url;

    /**
     * @var string
     */
    private $id;

    /**
     * @var string
     */
    private $comments;

    /**
     * @var AuditInfo
     */
    private $auditInfo;

    /**
     * @return AuditInfo
     */
    public function getAuditInfo()
    {
        return $this->auditInfo;
    }

    /**
     * @param AuditInfo $auditInfo
     */
    public function setAuditInfo(AuditInfo $auditInfo = null)
    {
        $this->auditInfo = $auditInfo;
    }


    /**
     * CustomerAccount constructor.
     * @param string $ID_CUSTOMER_ACCOUNT
     */
    public function __construct($ID_CUSTOMER_ACCOUNT = null)
    {
        if ($this->getPathParamData() === null) {
            $this->setPathParamData(new PathParamData());
        }
        $this->getPathParamData()->setIdCustomerAccount($ID_CUSTOMER_ACCOUNT);
        $this->id = $ID_CUSTOMER_ACCOUNT;
    }

    /**
     * @param string $account_holder_name
     * @param string $nickname
     * @param string $account_number
     * @param string $minimum_payment_due
     * @param string $past_amount_due
     * @param string $payment_due_date
     * @param string $statement_date
     * @return $this
     */
    public function withDetails($account_holder_name = null, $nickname = null, $account_number = null, $minimum_payment_due = null, $past_amount_due = null, $payment_due_date = null, $statement_date = null)
    {
        $this->account_holder_name = $account_holder_name;
        $this->nickname = $nickname;
        $this->account_number = $account_number;
        $this->minimum_payment_due = $minimum_payment_due;
        $this->past_amount_due = $past_amount_due;
        $this->payment_due_date = $payment_due_date;
        $this->statement_date = $statement_date;
        return $this;
    }

    /**
     * @param string $current_balance
     * @param string $current_statement_balance
     * @return $this
     */
    public function withBalances($current_balance = null, $current_statement_balance = null)
    {
        $this->current_balance = $current_balance;
        $this->current_statement_balance = $current_statement_balance;
        return $this;
    }

    /**
     * @param string $customer_account_reference
     * @return $this
     */
    public function withReference($customer_account_reference = null)
    {
        $this->customer_account_reference = $customer_account_reference;
        return $this;
    }

    /**
     * @param CustomerAccountStatus $status
     * @return $this
     */
    public function withStatus(CustomerAccountStatus $status = null)
    {
        $this->status = $status;
        return $this;
    }

    /**
     * @param Address $address
     * @return $this
     */
    public function withAddress(Address $address = null)
    {
        $this->address = $address;
        return $this;
    }

    /**
     * @param object $custom_fields
     * @return $this
     */
    public function withCustomFields($custom_fields = null)
    {
        if(ValidationUtil::isAssociativeArray($custom_fields)){
            $custom_fields = (object) $custom_fields;
        }
        $this->custom_fields = $custom_fields;
        return $this;
    }

    /**
     * @param string $ID_CUSTOMER
     * @return $this
     */
    public function forCustomer($ID_CUSTOMER = null)
    {
        if ($this->getPathParamData() === null) {
            $this->setPathParamData(new PathParamData());
        }
        $this->getPathParamData()->setIdCustomer($ID_CUSTOMER);
        return $this;
    }


    /**
     * @param string $client_key
     * @return $this
     */
    public function forClient($client_key = null)
    {
        parent::forClient($client_key);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function retrievedBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function renewedBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $channel
     * @return $this
     */
    public function videChannel($channel = null)
    {
        parent::videChannel($channel);
        return $this;
    }

    /**
     * @param string $url
     */
    public function setUrl($url)
    {
        $this->url = $url;
    }

    /**
     * @return string
     */
    public function getAccountHolderName()
    {
        return $this->account_holder_name;
    }

    /**
     * @return string
     */
    public function getNickname()
    {
        return $this->nickname;
    }

    /**
     * @return string
     */
    public function getAccountNumber()
    {
        return $this->account_number;
    }

    /**
     * @return CustomerAccountStatus
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @return string
     */
    public function getCustomerAccountReference()
    {
        return $this->customer_account_reference;
    }

    /**
     * @return string
     */
    public function getCurrentBalance()
    {
        return $this->current_balance;
    }

    /**
     * @return string
     */
    public function getComments()
    {
        return $this->comments;
    }

    /**
     * @return string
     */
    public function getCurrentStatementBalance()
    {
        return $this->current_statement_balance;
    }

    /**
     * @return string
     */
    public function getMinimumPaymentDue()
    {
        return $this->minimum_payment_due;
    }

    /**
     * @return string
     */
    public function getPastAmountDue()
    {
        return $this->past_amount_due;
    }

    /**
     * @return string
     */
    public function getPaymentDueDate()
    {
        return $this->payment_due_date;
    }

    /**
     * @return string
     */
    public function getStatementDate()
    {
        return $this->statement_date;
    }

    /**
     * @return Address
     */
    public function getAddress()
    {
        return $this->address;
    }

    /**
     * @return object
     */
    public function getCustomFields()
    {
        return $this->custom_fields;
    }

    /**
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * @return string
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return CustomerAccount
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function get(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new GetCustomerAccountHandler();
        return $handler->process($this);
    }

    public function renew(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new RenewCustomerAccountHandler();
        return $handler->process($this);
    }

    /**
     * @param string $comments
     * @return $this
     */
    public function withMemo($comments = null)
    {
        $this->comments = $comments;
        return $this;
    }

    /**
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        return CommonUtil::json_serialize(get_object_vars($this), parent::jsonSerialize());
    }
}