<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/21/18
 * Time: 1:27 PM
 */

namespace Orbipay\PaymentsApi\Client\Model\FundingAccount;


use Orbipay\PaymentsApi\Client\Enum\ACHEligibleFlag;
use Orbipay\PaymentsApi\Client\Enum\ATMEligibleFlag;
use Orbipay\PaymentsApi\Client\Enum\FundingAccountHolderType;
use Orbipay\PaymentsApi\Client\Enum\FundingAccountStatus;
use Orbipay\PaymentsApi\Client\Enum\FundingAccountSubtype;
use Orbipay\PaymentsApi\Client\Enum\FundingAccountType;
use Orbipay\PaymentsApi\Client\Enum\InvocationMode;
use Orbipay\PaymentsApi\Client\Enum\RequestorType;
use Orbipay\PaymentsApi\Client\Handler\FundingAccount\AddFundingAccountHandler;
use Orbipay\PaymentsApi\Client\Handler\FundingAccount\DeleteFundingAccountHandler;
use Orbipay\PaymentsApi\Client\Handler\FundingAccount\GetFundingAccountHandler;
use Orbipay\PaymentsApi\Client\Handler\FundingAccount\ReplaceFundingAccountHandler;
use Orbipay\PaymentsApi\Client\Handler\FundingAccount\UpdateFundingAccountHandler;
use Orbipay\PaymentsApi\Client\Model\Base\Address;
use Orbipay\PaymentsApi\Client\Model\Base\AuditInfo;
use Orbipay\PaymentsApi\Client\Model\Base\Base;
use Orbipay\PaymentsApi\Client\Model\Base\InvocationContext;
use Orbipay\PaymentsApi\Client\Model\Base\PathParamData;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;

class FundingAccount extends Base
{
    /**
     * @var ACHEligibleFlag
     */
    private $ach_eligible_flag;

    /**
     * @var string
     */
    private $card_cvv_number;

    /**
     * @var string
     */
    private $comments;

    /**
     * @var string
     */
    private $account_holder_name;

    /**
     * @var string
     */
    private $nickname;

    /**
     * @var string
     */
    private $account_number;

    /**
     * @var string
     */
    private $issuer_name;

    /**
     * @var FundingAccountStatus
     */
    private $status;

    /**
     * @var string
     */
    private $aba_routing_number;

    /**
     * @var FundingAccountSubtype
     */
    private $account_subtype;

    /**
     * @var string
     */
    private $currency_code3d;

    /**
     * @var string
     */
    private $expiry_date;

    /**
     * @var FundingAccountHolderType
     */
    private $account_holder_type;

    /**
     * @var Address
     */
    private $address;

    /**
     * @var ATMEligibleFlag
     */
    private $atm_eligible_flag;

    /**
     * @var string
     */
    private $id;

    /**
     * @var string
     */
    private $url;

    /**
     * @var string
     */
    private $deleted;

    /**
     * @var FundingAccountType
     */
    private $account_type;

    /**
     * @var object
     */
    private $custom_fields;

    /**
     * @var AuditInfo
     */
    private $auditInfo;

    /**
     * @return AuditInfo
     */
    public function getAuditInfo()
    {
        return $this->auditInfo;
    }

    /**
     * @param AuditInfo $auditInfo
     */
    public function setAuditInfo(AuditInfo $auditInfo = null)
    {
        $this->auditInfo = $auditInfo;
    }

    /**
     * Customer constructor.
     * @param string $ID_FUNDING_ACCOUNT
     */
    public function __construct($ID_FUNDING_ACCOUNT = null)
    {
        if ($this->getPathParamData() === null) {
            $this->setPathParamData(new PathParamData());
        }
        $this->getPathParamData()->setIdFundingAccount($ID_FUNDING_ACCOUNT);
        $this->id = $ID_FUNDING_ACCOUNT;
    }

    /**
     * @param string $account_holder_name
     * @param string $nickname
     * @param string $account_number
     * @param string $account_subtype
     * @param string $aba_routing_number
     * @param string $card_cvv_number
     * @param string $expiry_date
     * @param string $account_holder_type
     * @return FundingAccount
     */
    public function withDetails($account_holder_name = null, $nickname = null, $account_number = null, $account_subtype = null, $aba_routing_number = null, $card_cvv_number = null, $expiry_date = null, $account_holder_type = null)
    {
        $this->account_holder_name = $account_holder_name;
        $this->nickname = $nickname;
        $this->account_number = $account_number;
        $this->account_subtype = $account_subtype;
        $this->aba_routing_number = $aba_routing_number;
        $this->card_cvv_number = $card_cvv_number;
        $this->expiry_date = $expiry_date;
        $this->account_holder_type = $account_holder_type;
        return $this;
    }

    /**
     * @param string $ID_CUSTOMER
     * @return FundingAccount
     */
    public function forCustomer($ID_CUSTOMER = null)
    {
        if ($this->getPathParamData() === null) {
            $this->setPathParamData(new PathParamData());
        }
        $this->getPathParamData()->setIdCustomer($ID_CUSTOMER);
        return $this;
    }

    /**
     * @param FundingAccountStatus $status
     * @return $this
     */
    public function withStatus(FundingAccountStatus $status = null)
    {
        $this->status = $status;
        return $this;
    }

    /**
     * @param Address $address
     * @return $this
     */
    public function withAddress(Address $address = null)
    {
        $this->address = $address;
        return $this;
    }

    /**
     * @param object $custom_fields
     * @return $this
     */
    public function withCustomFields($custom_fields = null)
    {
        if(ValidationUtil::isAssociativeArray($custom_fields)){
            $custom_fields = (object) $custom_fields;
        }
        $this->custom_fields = $custom_fields;
        return $this;
    }

    /**
     * @param string $comments
     * @return $this
     */
    public function withMemo($comments = null)
    {
        $this->comments = $comments;
        return $this;
    }

    /**
     * @param string $client_key
     * @return $this
     */
    public function forClient($client_key = null)
    {
        parent::forClient($client_key);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function createdBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function updatedBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function replacedBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function retrievedBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function deletedBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $channel
     * @return $this
     */
    public function videChannel($channel = null)
    {
        parent::videChannel($channel);
        return $this;
    }

    /**
     * @internal
     * @param ACHEligibleFlag $ach_eligible_flag
     */
    public function setAchEligibleFlag(ACHEligibleFlag $ach_eligible_flag = null)
    {
        $this->ach_eligible_flag = $ach_eligible_flag;
    }

    /**
     * @param string $issuer_name
     */
    public function setIssuerName($issuer_name = null)
    {
        $this->issuer_name = $issuer_name;
    }

    /**
     * @param ATMEligibleFlag $atm_eligible_flag
     */
    public function setAtmEligibleFlag(ATMEligibleFlag $atm_eligible_flag = null)
    {
        $this->atm_eligible_flag = $atm_eligible_flag;
    }

    /**
     * @internal
     * @param string $currency_code3d
     */
    public function setCurrencyCode3d($currency_code3d = null)
    {
        $this->currency_code3d = $currency_code3d;
    }

    /**
     * @internal
     * @param string $url
     */
    public function setUrl($url = null)
    {
        $this->url = $url;
    }

    /**
     * @internal
     * @param string $deleted
     */
    public function setDeleted($deleted = null)
    {
        $this->deleted = $deleted;
    }

    /**
     * @internal
     * @param FundingAccountType $account_type
     */
    public function setAccountType(FundingAccountType $account_type = null)
    {
        $this->account_type = $account_type;
    }

    /**
     * @return ACHEligibleFlag
     */
    public function getAchEligibleFlag()
    {
        return $this->ach_eligible_flag;
    }

    /**
     * @return string
     */
    public function getCardCvvNumber()
    {
        return $this->card_cvv_number;
    }

    /**
     * @return string
     */
    public function getComments()
    {
        return $this->comments;
    }

    /**
     * @return string
     */
    public function getAccountHolderName()
    {
        return $this->account_holder_name;
    }

    /**
     * @return string
     */
    public function getNickname()
    {
        return $this->nickname;
    }

    /**
     * @return string
     */
    public function getAccountNumber()
    {
        return $this->account_number;
    }

    /**
     * @return string
     */
    public function getIssuerName()
    {
        return $this->issuer_name;
    }

    /**
     * @return FundingAccountStatus
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @return string
     */
    public function getAbaRoutingNumber()
    {
        return $this->aba_routing_number;
    }

    /**
     * @return FundingAccountSubtype
     */
    public function getAccountSubtype()
    {
        return $this->account_subtype;
    }

    /**
     * @return string
     */
    public function getCurrencyCode3d()
    {
        return $this->currency_code3d;
    }

    /**
     * @return string
     */
    public function getExpiryDate()
    {
        return $this->expiry_date;
    }

    /**
     * @return FundingAccountHolderType
     */
    public function getAccountHolderType()
    {
        return $this->account_holder_type;
    }

    /**
     * @return Address
     */
    public function getAddress()
    {
        return $this->address;
    }

    /**
     * @return ATMEligibleFlag
     */
    public function getAtmEligibleFlag()
    {
        return $this->atm_eligible_flag;
    }

    /**
     * @return string
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * @return string
     */
    public function getDeleted()
    {
        return $this->deleted;
    }

    /**
     * @return FundingAccountType
     */
    public function getAccountType()
    {
        return $this->account_type;
    }

    /**
     * @return object
     */
    public function getCustomFields()
    {
        return $this->custom_fields;
    }

    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return FundingAccount
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function create(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new AddFundingAccountHandler();
        return $handler->process($this);
    }

    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return FundingAccount
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function update(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new UpdateFundingAccountHandler();
        return $handler->process($this);
    }

    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return FundingAccount
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function get(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new GetFundingAccountHandler();
        return $handler->process($this);
    }


    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return FundingAccount
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function delete(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new DeleteFundingAccountHandler();
        return $handler->process($this);
    }

    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return FundingAccount
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function replace(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new ReplaceFundingAccountHandler();
        return $handler->process($this);
    }


    /**
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        return CommonUtil::json_serialize(get_object_vars($this), parent::jsonSerialize());
    }
}