<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/20/18
 * Time: 3:17 PM
 */

namespace Orbipay\PaymentsApi\Client\Model\FundingAccount;

use Iterator;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Enum\HasMoreResults;
use Orbipay\PaymentsApi\Client\Enum\InvocationMode;
use Orbipay\PaymentsApi\Client\Enum\RequestorType;
use Orbipay\PaymentsApi\Client\Handler\FundingAccount\RetrieveFundingAccountsHandler;
use Orbipay\PaymentsApi\Client\Model\Base\Base;
use Orbipay\PaymentsApi\Client\Model\Base\FormParamData;
use Orbipay\PaymentsApi\Client\Model\Base\InvocationContext;
use Orbipay\PaymentsApi\Client\Model\Base\MultiMap;
use Orbipay\PaymentsApi\Client\Model\Base\PathParamData;
use Orbipay\PaymentsApi\Client\Model\Base\QueryParamData;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;

class FundingAccounts extends Base implements Iterator
{
    private $position = 0;

    /**
     * @var FundingAccount[]
     */
    private $list;

    /**
     * @var string
     */
    private $total_results_count;

    /**
     * @var string
     */
    private $hasMoreResults;

    /**
     * @var string
     */
    private $url;

    /**
     * @var string
     */
    private $fromIndex;

    /**
     * @var string
     */
    private $toIndex;

    /**
     * @var string
     */
    private $queryId;


    /**
     * Customers constructor.
     */
    public function __construct()
    {
    }

    /**
     * @param string $client_key
     * @return $this
     */
    public function forClient($client_key = null)
    {
        parent::forClient($client_key);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function retrievedBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $channel
     * @return $this
     */
    public function videChannel($channel = null)
    {
        parent::videChannel($channel);
        return $this;
    }

    /**
     * @param MultiMap $filter
     * @return $this
     */
    public function findBy(MultiMap $filter = null)
    {
        if ($this->getFormParamData() === null) {
            $this->setFormParamData(new FormParamData());
        }

        if ($this->getQueryParamData() === null) {
            $this->setQueryParamData(new QueryParamData());
        }

        if ($filter != null) {

            $account_number_values = $filter->get(HttpParamConstants::ACCOUNT_NUMBER);
            if ($account_number_values !== null && count($account_number_values) > 0) {
                $this->getFormParamData()->setAccountNumber(end($account_number_values));
            }

            $status_values = $filter->get(HttpParamConstants::STATUS);
            if ($status_values !== null) {
                $this->getFormParamData()->setStatus(array_unique($status_values));
            }

            $aba_routing_number_values = $filter->get(HttpParamConstants::ABA_ROUTING_NUMBER);
            if ($aba_routing_number_values !== null) {
                $this->getFormParamData()->setAbaRoutingNumber(end($aba_routing_number_values));
            }

            $account_subtype_values = $filter->get(HttpParamConstants::ACCOUNT_SUBTYPE);
            if ($account_subtype_values !== null) {
                $this->getFormParamData()->setAccountSubtype(array_unique($account_subtype_values));
            }

            $account_type_values = $filter->get(HttpParamConstants::ACCOUNT_TYPE);
            if ($account_type_values !== null) {
                $this->getFormParamData()->setAccountType(array_unique($account_type_values));
            }

            $page_size_values = $filter->get(HttpParamConstants::PAGE_SIZE);
            if ($page_size_values !== null && count($page_size_values) > 0) {
                $this->getQueryParamData()->setPageSize(end($page_size_values));
            }

            $query_id_values = $filter->get(HttpParamConstants::QUERY_ID);
            if ($query_id_values !== null && count($query_id_values) > 0) {
                $this->getQueryParamData()->setQueryId(end($query_id_values));
            }

            $from_index_values = $filter->get(HttpParamConstants::FROM_INDEX);
            if ($from_index_values !== null && count($from_index_values) > 0) {
                $this->getQueryParamData()->setFromIndex(end($from_index_values));
            }

            $to_index_values = $filter->get(HttpParamConstants::TO_INDEX);
            if ($to_index_values !== null && count($to_index_values) > 0) {
                $this->getQueryParamData()->setToIndex(end($to_index_values));
            }

        }
        return $this;
    }

    /**
     * @param string $ID_CUSTOMER
     * @return $this
     */
    public function forCustomer($ID_CUSTOMER = null)
    {
        if ($this->getPathParamData() === null) {
            $this->setPathParamData(new PathParamData());
        }
        $this->getPathParamData()->setIdCustomer($ID_CUSTOMER);
        return $this;
    }

    /**
     * @return FundingAccount[]
     */
    public function getList()
    {
        return $this->list;
    }

    /**
     * @return string
     */
    public function getTotalResultsCount()
    {
        return $this->total_results_count;
    }

    /**
     * @return string
     */
    public function getHasMoreResults()
    {
        return $this->hasMoreResults;
    }

    /**
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * @return string
     */
    public function getFromIndex()
    {
        return $this->fromIndex;
    }

    /**
     * @return string
     */
    public function getToIndex()
    {
        return $this->toIndex;
    }

    /**
     * @return string
     */
    public function getQueryId()
    {
        return $this->queryId;
    }

    /**
     * @internal
     * @param FundingAccount[] $list
     */
    public function setList($list = null)
    {
        $this->list = $list;
    }

    /**
     * @internal
     * @param string $total_results_count
     */
    public function setTotalResultsCount($total_results_count = null)
    {
        $this->total_results_count = $total_results_count;
    }

    /**
     * @internal
     * @param string $hasMoreResults
     */
    public function setHasMoreResults($hasMoreResults = null)
    {
        $this->hasMoreResults = $hasMoreResults;
    }

    /**
     * @internal
     * @param string $url
     */
    public function setUrl($url = null)
    {
        $this->url = $url;
    }

    /**
     * @internal
     * @param string $fromIndex
     */
    public function setFromIndex($fromIndex = null)
    {
        $this->fromIndex = $fromIndex;
    }

    /**
     * @internal
     * @param string $toIndex
     */
    public function setToIndex($toIndex = null)
    {
        $this->toIndex = $toIndex;
    }

    /**
     * @internal
     * @param string $queryId
     */
    public function setQueryId($queryId = null)
    {
        $this->queryId = $queryId;
    }

    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return FundingAccounts
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function get(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new RetrieveFundingAccountsHandler();
        return $handler->process($this);
    }

    /**
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        return CommonUtil::json_serialize(get_object_vars($this), parent::jsonSerialize());
    }

    /**
     * Return the current element
     * @link http://php.net/manual/en/iterator.current.php
     * @return FundingAccount
     * @since 5.0.0
     */
    public function current()
    {
        return $this->list[$this->position];
    }

    /**
     * Move forward to next element
     * @link http://php.net/manual/en/iterator.next.php
     * @return void Any returned value is ignored.
     * @since 5.0.0
     */
    public function next()
    {
        ++$this->position;
    }

    /**
     * Return the key of the current element
     * @link http://php.net/manual/en/iterator.key.php
     * @return mixed scalar on success, or null on failure.
     * @since 5.0.0
     */
    public function key()
    {
        return $this->position;
    }

    /**
     * Checks if current position is valid
     * @link http://php.net/manual/en/iterator.valid.php
     * @return boolean The return value will be casted to boolean and then evaluated.
     * Returns true on success or false on failure.
     * @since 5.0.0
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function valid()
    {
        $valid = false;
        if ($this->position < count($this->list)) {
            $valid = true;
        } elseif ($this->hasMoreResults === HasMoreResults::TRUE) {
            $this->populateQueryInfo();
            $this->get($this->getInvocationContext(), $this->getInvocationMode(), $this->getApiEndPoint());
            $this->position = 0;
            $valid = true;
        }
        return $valid;
    }

    public function populateQueryInfo()
    {
        if ($this->getQueryParamData() === null) {
            $this->setQueryParamData(new QueryParamData());
        }

        $queryParamData = $this->getQueryParamData();
        $queryParamData->setQueryId($this->getQueryId());
        $queryParamData->setFromIndex((string) (intval($this->getToIndex()) + 1));
        $queryParamData->setToIndex(null);
    }


    /**
     * Rewind the Iterator to the first element
     * @link http://php.net/manual/en/iterator.rewind.php
     * @return void Any returned value is ignored.
     * @since 5.0.0
     */
    public function rewind()
    {
        /*$this->position = 0;*/
        // Actually rewinding would require making a copy of the original page. So rewinding is not provided.
    }
}