<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/20/18
 * Time: 3:18 PM
 */

namespace Orbipay\PaymentsApi\Client\Model\Payment;

use Iterator;
use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Enum\HasMoreResults;
use Orbipay\PaymentsApi\Client\Enum\InvocationMode;
use Orbipay\PaymentsApi\Client\Enum\RequestorType;
use Orbipay\PaymentsApi\Client\Handler\Payment\RetrievePaymentsHandler;
use Orbipay\PaymentsApi\Client\Model\Base\Base;
use Orbipay\PaymentsApi\Client\Model\Base\InvocationContext;
use Orbipay\PaymentsApi\Client\Model\Base\MultiMap;
use Orbipay\PaymentsApi\Client\Model\Base\QueryParamData;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;

class Payments extends Base implements Iterator
{
    private $position = 0;

    /**
     * @var Payment[]
     */
    private $list = [];

    /**
     * @var string
     */
    private $total_results_count;

    /**
     * @var string
     */
    private $hasMoreResults;

    /**
     * @var string
     */
    private $url;

    /**
     * @var string
     */
    private $fromIndex;

    /**
     * @var string
     */
    private $toIndex;

    /**
     * @var string
     */
    private $queryId;


    /**
     * Customers constructor.
     */
    public function __construct()
    {
    }

    /**
     * @param string $client_key
     * @return $this
     */
    public function forClient($client_key = null)
    {
        parent::forClient($client_key);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function retrievedBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $channel
     * @return $this
     */
    public function videChannel($channel = null)
    {
        parent::videChannel($channel);
        return $this;
    }

    /**
     * @param MultiMap $filter
     * @return $this
     */
    public function findBy(MultiMap $filter = null)
    {
        if ($this->getQueryParamData() === null) {
            $this->setQueryParamData(new QueryParamData());
        }

        if ($filter != null) {

            $id_customer_values = $filter->get(HttpParamConstants::QUERY_PARAM_ID_CUSTOMER);
            if ($id_customer_values !== null && count($id_customer_values) > 0) {
                $this->getQueryParamData()->setIdCustomer(end($id_customer_values));
            }

            $id_customer_account_values = $filter->get(HttpParamConstants::QUERY_PARAM_ID_CUSTOMER_ACCOUNT);
            if ($id_customer_account_values !== null && count($id_customer_account_values) > 0) {
                $this->getQueryParamData()->setIdCustomerAccount(end($id_customer_account_values));
            }

            $id_funding_account_values = $filter->get(HttpParamConstants::QUERY_PARAM_ID_FUNDING_ACCOUNT);
            if ($id_funding_account_values !== null && count($id_funding_account_values) > 0) {
                $this->getQueryParamData()->setIdFundingAccount(end($id_funding_account_values));
            }

            $confirmation_number_values = $filter->get(HttpParamConstants::CONFIRMATION_NUMBER);
            if ($confirmation_number_values !== null && count($confirmation_number_values) > 0) {
                $this->getQueryParamData()->setConfirmationNumber(end($confirmation_number_values));
            }

            $from_date_values = $filter->get(HttpParamConstants::FROM_DATE);
            if ($from_date_values !== null && count($from_date_values) > 0) {
                $this->getQueryParamData()->setFromDate(end($from_date_values));
            }

            $to_date_values = $filter->get(HttpParamConstants::TO_DATE);
            if ($to_date_values !== null && count($to_date_values) > 0) {
                $this->getQueryParamData()->setToDate(end($to_date_values));
            }

            $status_values = $filter->get(HttpParamConstants::STATUS);
            if ($status_values !== null) {
                $this->getQueryParamData()->setStatus(array_unique($status_values));
            }

            $payment_schedule_values = $filter->get(HttpParamConstants::PAYMENT_SCHEDULE_TYPE);
            if ($payment_schedule_values !== null) {
                $this->getQueryParamData()->setPaymentScheduleType(array_unique($payment_schedule_values));
            }

            $_txn_pymnt_schld_type = $filter->get(HttpParamConstants::_TXN_PYMNT_SCHLD_TYPE);
            if ($_txn_pymnt_schld_type !== null) {
                $this->getQueryParamData()->setTxnPymntSchldType(array_unique($_txn_pymnt_schld_type));
            }

            $page_size_values = $filter->get(HttpParamConstants::PAGE_SIZE);
            if ($page_size_values !== null && count($page_size_values) > 0) {
                $this->getQueryParamData()->setPageSize(end($page_size_values));
            }

            $query_id_values = $filter->get(HttpParamConstants::QUERY_ID);
            if ($query_id_values !== null && count($query_id_values) > 0) {
                $this->getQueryParamData()->setQueryId(end($query_id_values));
            }

            $from_index_values = $filter->get(HttpParamConstants::FROM_INDEX);
            if ($from_index_values !== null && count($from_index_values) > 0) {
                $this->getQueryParamData()->setFromIndex(end($from_index_values));
            }

            $to_index_values = $filter->get(HttpParamConstants::TO_INDEX);
            if ($to_index_values !== null && count($to_index_values) > 0) {
                $this->getQueryParamData()->setToIndex(end($to_index_values));
            }

        }
        return $this;
    }

    /**
     * @param string $ID_CUSTOMER
     * @return $this
     */
    public function forCustomer($ID_CUSTOMER = null)
    {
        if ($this->getQueryParamData() === null) {
            $this->setQueryParamData(new QueryParamData());
        }
        $this->getQueryParamData()->setIdCustomer($ID_CUSTOMER);
        return $this;
    }

    /**
     * @return Payment[]
     */
    public function getList()
    {
        return $this->list;
    }

    /**
     * @return string
     */
    public function getTotalResultsCount()
    {
        return $this->total_results_count;
    }

    /**
     * @return string
     */
    public function getHasMoreResults()
    {
        return $this->hasMoreResults;
    }

    /**
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * @return string
     */
    public function getFromIndex()
    {
        return $this->fromIndex;
    }

    /**
     * @return string
     */
    public function getToIndex()
    {
        return $this->toIndex;
    }

    /**
     * @return string
     */
    public function getQueryId()
    {
        return $this->queryId;
    }

    /**
     * @internal
     * @param Payment[] $list
     */
    public function setList($list = null)
    {
        $this->list = $list;
    }

    /**
     * @internal
     * @param string $total_results_count
     */
    public function setTotalResultsCount($total_results_count = null)
    {
        $this->total_results_count = $total_results_count;
    }

    /**
     * @internal
     * @param string $hasMoreResults
     */
    public function setHasMoreResults($hasMoreResults = null)
    {
        $this->hasMoreResults = $hasMoreResults;
    }

    /**
     * @internal
     * @param string $url
     */
    public function setUrl($url = null)
    {
        $this->url = $url;
    }

    /**
     * @internal
     * @param string $fromIndex
     */
    public function setFromIndex($fromIndex = null)
    {
        $this->fromIndex = $fromIndex;
    }

    /**
     * @internal
     * @param string $toIndex
     */
    public function setToIndex($toIndex = null)
    {
        $this->toIndex = $toIndex;
    }

    /**
     * @internal
     * @param string $queryId
     */
    public function setQueryId($queryId = null)
    {
        $this->queryId = $queryId;
    }

    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return Payments
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function get(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new RetrievePaymentsHandler();
        return $handler->process($this);
    }

    /**
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        return CommonUtil::json_serialize(get_object_vars($this), parent::jsonSerialize());
    }

    /**
     * Return the current element
     * @link http://php.net/manual/en/iterator.current.php
     * @return Payment
     * @since 5.0.0
     */
    public function current()
    {
        return $this->list[$this->position];
    }

    /**
     * Move forward to next element
     * @link http://php.net/manual/en/iterator.next.php
     * @return void Any returned value is ignored.
     * @since 5.0.0
     */
    public function next()
    {
        ++$this->position;
    }

    /**
     * Return the key of the current element
     * @link http://php.net/manual/en/iterator.key.php
     * @return mixed scalar on success, or null on failure.
     * @since 5.0.0
     */
    public function key()
    {
        return $this->position;
    }

    /**
     * Checks if current position is valid
     * @link http://php.net/manual/en/iterator.valid.php
     * @return boolean The return value will be casted to boolean and then evaluated.
     * Returns true on success or false on failure.
     * @since 5.0.0
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function valid()
    {
        $valid = false;
        if ($this->position < count($this->list)) {
            $valid = true;
        } elseif ($this->hasMoreResults === HasMoreResults::TRUE) {
            $this->populateQueryInfo();
            $this->get($this->getInvocationContext(), $this->getInvocationMode(), $this->getApiEndPoint());
            $this->position = 0;
            $valid = true;
        }
        return $valid;
    }

    public function populateQueryInfo()
    {
        if ($this->getQueryParamData() === null) {
            $this->setQueryParamData(new QueryParamData());
        }

        $queryParamData = $this->getQueryParamData();
        $queryParamData->setQueryId($this->getQueryId());
        $queryParamData->setFromIndex((string) (intval($this->getToIndex()) + 1));
        $queryParamData->setToIndex(null);
    }


    /**
     * Rewind the Iterator to the first element
     * @link http://php.net/manual/en/iterator.rewind.php
     * @return void Any returned value is ignored.
     * @since 5.0.0
     */
    public function rewind()
    {
        $this->position = 0;
    }
}
