<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/21/18
 * Time: 3:45 PM
 */

namespace Orbipay\PaymentsApi\Client\Model\Payment;


use Orbipay\PaymentsApi\Client\Enum\InvocationMode;
use Orbipay\PaymentsApi\Client\Enum\PaymentAmountType;
use Orbipay\PaymentsApi\Client\Enum\PaymentMethod;
use Orbipay\PaymentsApi\Client\Enum\PaymentScheduleType;
use Orbipay\PaymentsApi\Client\Enum\PaymentStatus;
use Orbipay\PaymentsApi\Client\Enum\RequestorType;
use Orbipay\PaymentsApi\Client\Handler\Payment\CreateSimplePaymentHandler;
use Orbipay\PaymentsApi\Client\Model\Base\Base;
use Orbipay\PaymentsApi\Client\Model\Base\InvocationContext;
use Orbipay\PaymentsApi\Client\Model\Base\QueryParamData;
use Orbipay\PaymentsApi\Client\Model\Customer\Customer;
use Orbipay\PaymentsApi\Client\Model\CustomerAccount\CustomerAccount;
use Orbipay\PaymentsApi\Client\Model\FundingAccount\FundingAccount;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;

class SimplePayment extends Base
{
    /**
     * @var string
     */
    private $payment_reference;

    /**
     * @var string
     */
    private $id;

    /**
     * @var string
     */
    private $amount;

    /**
     * @var string
     */
    private $currency_code3d;

    /**
     * @var string
     */
    private $payment_date;

    /**
     * @var Fee
     */
    private $fee;

    /**
     * @var Customer
     */
    private $customer;

    /**
     * @var string
     */
    private $card_cvv_number;

    /**
     * @var FundingAccount
     */
    private $funding_account;

    /**
     * @var CustomerAccount
     */
    private $customer_account;

    /**
     * @var object
     */
    private $custom_fields;

    /**
     * @var PaymentStatus
     */
    private $status;

    /**
     * @var string
     */
    private $confirmation_number;

    /**
     * @var string
     */
    private $comments;

    /**
     * @var string
     */
    private $deleted;

    /**
     * @var string
     */
    private $payment_entry_date;

    /**
     * @var string
     */
    private $Payment_return_date;

    /**
     * @var string
     */
    private $return_code;

    /**
     * @var PaymentMethod
     */
    private $payment_method;

    /**
     * @var PaymentScheduleType
     */
    private $payment_schedule_type;

    /**
     * @var string
     */
    private $url;

    /**
     * @var PaymentAmountType
     */
    private $payment_amount_type;

    /**
     * @var string
     */
    private $payment_request_date;

    /**
     * @var string
     */
    private $expected_payment_settlement_date;


    /**
     * @var PaymentNetworkResponse
     */
    private $payment_network_response;

    /**
     * @var AuditInfo
     */
    private $auditInfo;

    /**
     * @return AuditInfo
     */
    public function getAuditInfo()
    {
        return $this->auditInfo;
    }

    /**
     * @param AuditInfo $auditInfo
     */
    public function setAuditInfo(AuditInfo $auditInfo = null)
    {
        $this->auditInfo = $auditInfo;
    }

    /**
     * @return PaymentNetworkResponse
     */
    public function getPaymentNetworkResponse()
    {
        return $this->payment_network_response;
    }

    /**
     * @param PaymentNetworkResponse $payment_network_response
     */
    public function setPaymentNetworkResponse($payment_network_response)
    {
        $this->payment_network_response = $payment_network_response;
    }

    /**
     * @param PaymentAmountType $payment_amount_type
     * @param string $amount
     * @param string $payment_date
     * @param string $payment_request_date
     * @param string $card_cvv_number
     * @return $this
     */
    public function withDetails(PaymentAmountType $payment_amount_type = null, $amount = null, $payment_date = null, $payment_request_date = null, $card_cvv_number = null)
    {
        $this->amount = $amount;
        $this->payment_date = $payment_date;
        $this->card_cvv_number = $card_cvv_number;
        $this->payment_amount_type = $payment_amount_type;
        $this->payment_request_date = $payment_request_date;
        return $this;
    }

    /**
     * @param string $ID_CUSTOMER
     * @return $this
     */
    public function forCustomer(Customer $customer)
    {
        $this->customer = $customer;
        return $this;
    }

    /**
     * @param FundingAccount $funding_account
     * @return $this
     */
    public function fromAccount(FundingAccount $funding_account = null, $save_funding_account = null)
    {
        $this->funding_account = $funding_account;
        if ($this->getQueryParamData() === null) {
            $this->setQueryParamData(new QueryParamData());
        }
        $this -> getQueryParamData()->setSaveFundingAccount($save_funding_account);
        return $this;
    }

    /**
     * @param CustomerAccount $customer_acccount
     * @return $this
     */
    public function toAccount(CustomerAccount $customer_account = null)
    {
        $this->customer_account = $customer_account;
        return $this;
    }

    /**
     * @param PaymentStatus $status
     * @return $this
     */
    public function withStatus(PaymentStatus $status = null)
    {
        $this->status = $status;
        return $this;
    }

    /**
     * @param object $custom_fields
     * @return $this
     */
    public function withCustomFields($custom_fields = null)
    {
        if(ValidationUtil::isAssociativeArray($custom_fields)){
            $custom_fields = (object) $custom_fields;
        }
        $this->custom_fields = $custom_fields;
        return $this;
    }

    /**
     * @param string $payment_reference
     * @return $this
     */
    public function withReference($payment_reference = null)
    {
        $this->payment_reference = $payment_reference;
        return $this;
    }

    /**
     * @param string $comments
     * @return $this
     */
    public function withMemo($comments = null)
    {
        $this->comments = $comments;
        return $this;
    }

    /**
     * @param string $client_key
     * @return $this
     */
    public function forClient($client_key = null)
    {
        parent::forClient($client_key);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function createdBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $channel
     * @return $this
     */
    public function videChannel($channel = null)
    {
        parent::videChannel($channel);
        return $this;
    }

    /**
     * @param Fee $fee
     * @return $this
     */
    public function withFee(Fee $fee = null)
    {
        $this->fee = $fee;
        return $this;
    }

    /**
     * @internal
     * @param string $currency_code3d
     */
    public function setCurrencyCode3d($currency_code3d = null)
    {
        $this->currency_code3d = $currency_code3d;
    }

    /**
     * @internal
     * @param string $confirmation_number
     */
    public function setConfirmationNumber($confirmation_number = null)
    {
        $this->confirmation_number = $confirmation_number;
    }

    /**
     * @internal
     * @param string $payment_entry_date
     */
    public function setPaymentEntryDate($payment_entry_date = null)
    {
        $this->payment_entry_date = $payment_entry_date;
    }

    /**
     * @internal
     * @param string $Payment_return_date
     */
    public function setPaymentReturnDate($Payment_return_date = null)
    {
        $this->Payment_return_date = $Payment_return_date;
    }

    /**
     * @internal
     * @param string $return_code
     */
    public function setReturnCode($return_code = null)
    {
        $this->return_code = $return_code;
    }

    /**
     * @internal
     * @param PaymentMethod $payment_method
     */
    public function setPaymentMethod(PaymentMethod $payment_method = null)
    {
        $this->payment_method = $payment_method;
    }

    /**
     * @internal
     * @param PaymentScheduleType $payment_schedule_type
     */
    public function setPaymentScheduleType(PaymentScheduleType $payment_schedule_type = null)
    {
        $this->payment_schedule_type = $payment_schedule_type;
    }

    /**
     * @internal
     * @param string $url
     */
    public function setUrl($url = null)
    {
        $this->url = $url;
    }

    /**
     * @internal
     * @param string $expected_payment_settlement_date
     */
    public function setExpectedPaymentSettlementDate($expected_payment_settlement_date)
    {
        $this->expected_payment_settlement_date = $expected_payment_settlement_date;
    }

    /**
     * @internal
     * @param Fee $fee
     */
    public function setFee(Fee $fee = null)
    {
        $this->fee = $fee;
    }

    /**
     * @return string
     */
    public function getPaymentReference()
    {
        return $this->payment_reference;
    }

    /**
     * @return string
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return string
     */
    public function getAmount()
    {
        return $this->amount;
    }

    /**
     * @return string
     */
    public function getCurrencyCode3d()
    {
        return $this->currency_code3d;
    }

    /**
     * @return string
     */
    public function getPaymentDate()
    {
        return $this->payment_date;
    }

    /**
     * @return Fee
     */
    public function getFee()
    {
        return $this->fee;
    }

    /**
     * @return Customer
     */
    public function getCustomer()
    {
        return $this->customer;
    }

    /**
     * @return CustomerAccount
     */
    public function getCustomerAccount()
    {
        return $this->customer_account;
    }

    /**
     * @return FundingAccount
     */
    public function getFundingAccount()
    {
        return $this->funding_account;
    }



    /**
     * @return string
     */
    public function getCardCvvNumber()
    {
        return $this->card_cvv_number;
    }

    /**
     * @return object
     */
    public function getCustomFields()
    {
        return $this->custom_fields;
    }

    /**
     * @return PaymentStatus
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @return string
     */
    public function getConfirmationNumber()
    {
        return $this->confirmation_number;
    }

    /**
     * @return string
     */
    public function getComments()
    {
        return $this->comments;
    }

    /**
     * @return string
     */
    public function getDeleted()
    {
        return $this->deleted;
    }

    /**
     * @return string
     */
    public function getPaymentEntryDate()
    {
        return $this->payment_entry_date;
    }

    /**
     * @return string
     */
    public function getPaymentReturnDate()
    {
        return $this->Payment_return_date;
    }

    /**
     * @return string
     */
    public function getReturnCode()
    {
        return $this->return_code;
    }

    /**
     * @return PaymentMethod
     */
    public function getPaymentMethod()
    {
        return $this->payment_method;
    }

    /**
     * @return PaymentScheduleType
     */
    public function getPaymentScheduleType()
    {
        return $this->payment_schedule_type;
    }

    /**
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * @return PaymentAmountType
     */
    public function getPaymentAmountType()
    {
        return $this->payment_amount_type;
    }

    /**
     * @return string
     */
    public function getPaymentRequestDate()
    {
        return $this->payment_request_date;
    }

    /**
     * @return string
     */
    public function getExpectedPaymentSettlementDate()
    {
        return $this->expected_payment_settlement_date;
    }

    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return Payment
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function create(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new CreateSimplePaymentHandler();
        return $handler->process($this);
    }


    /**
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        return CommonUtil::json_serialize(get_object_vars($this), parent::jsonSerialize());
    }
}