<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/23/18
 * Time: 12:49 PM
 */

namespace Orbipay\PaymentsApi\Client\Model\PaymentSetup;


use Orbipay\PaymentsApi\Client\Enum\PaymentAmountType;
use Orbipay\PaymentsApi\Client\Enum\PaymentRecurringType;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;

class PaymentSchedule implements \JsonSerializable
{
    /**
     * @var PaymentRecurringType
     */
    private $payment_recurring_type;

    /**
     * @var string
     */
    private $payment_recurring_count;

    /**
     * @var PaymentAmountType
     */
    private $payment_amount_type;

    /**
     * @var string
     */
    private $payment_start_date;

    /**
     * @var string
     */
    private $payment_end_date;

    /**
     * @var string
     */
    private $payment_limit_amount;

    /**
     * @var string
     */
    private $payment_plan_id;

    /**
     * @var string
     */
    private $payment_defer_days;

    /**
     * PaymentSchedule constructor.
     * @param PaymentRecurringType $payment_recurring_type
     * @param string $payment_recurring_count
     * @param PaymentAmountType $payment_amount_type
     * @param string $payment_start_date
     * @param string $payment_end_date
     * @param string $payment_limit_amount
     * @param string $payment_plan_id
     * @param string $payment_defer_days
     */
    public function __construct(PaymentRecurringType $payment_recurring_type = null, $payment_recurring_count = null, PaymentAmountType $payment_amount_type = null, $payment_start_date = null, $payment_end_date = null, $payment_limit_amount = null, $payment_plan_id = null, $payment_defer_days = null)
    {
        $this->payment_recurring_type = $payment_recurring_type;
        $this->payment_recurring_count = $payment_recurring_count;
        $this->payment_amount_type = $payment_amount_type;
        $this->payment_start_date = $payment_start_date;
        $this->payment_end_date = $payment_end_date;
        $this->payment_limit_amount = $payment_limit_amount;
        $this->payment_plan_id = $payment_plan_id;
        $this->payment_defer_days = $payment_defer_days;
    }

    /**
     * @return PaymentRecurringType
     */
    public function getPaymentRecurringType()
    {
        return $this->payment_recurring_type;
    }

    /**
     * @return string
     */
    public function getPaymentRecurringCount()
    {
        return $this->payment_recurring_count;
    }

    /**
     * @return PaymentAmountType
     */
    public function getPaymentAmountType()
    {
        return $this->payment_amount_type;
    }

    /**
     * @return string
     */
    public function getPaymentStartDate()
    {
        return $this->payment_start_date;
    }

    /**
     * @return string
     */
    public function getPaymentEndDate()
    {
        return $this->payment_end_date;
    }

    /**
     * @return string
     */
    public function getPaymentLimitAmount()
    {
        return $this->payment_limit_amount;
    }

    /**
     * @return string
     */
    public function getPaymentPlanId()
    {
        return $this->payment_plan_id;
    }

    /**
     * @return string
     */
    public function getPaymentDeferDays()
    {
        return $this->payment_defer_days;
    }

    /**
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        return CommonUtil::json_serialize(get_object_vars($this));
    }



    /**
     * @param string $payment_limit_amount
     * @return $this
     */
    public function withPaymentLimitAmount($payment_limit_amount = null)
    {
        $this->payment_limit_amount = $payment_limit_amount;
        return $this;
    }
}
