<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/21/18
 * Time: 3:46 PM
 */

namespace Orbipay\PaymentsApi\Client\Model\PaymentSetup;


use Orbipay\PaymentsApi\Client\Enum\InvocationMode;
use Orbipay\PaymentsApi\Client\Enum\PaymentMethod;
use Orbipay\PaymentsApi\Client\Enum\PaymentSetupScheduleType;
use Orbipay\PaymentsApi\Client\Enum\PaymentSetupStatus;
use Orbipay\PaymentsApi\Client\Enum\RequestorType;
use Orbipay\PaymentsApi\Client\Handler\PaymentSetup\DeletePaymentSetupHandler;
use Orbipay\PaymentsApi\Client\Handler\PaymentSetup\GetPaymentSetupHandler;
use Orbipay\PaymentsApi\Client\Model\Base\AuditInfo;
use Orbipay\PaymentsApi\Client\Model\Base\Base;
use Orbipay\PaymentsApi\Client\Model\Base\InvocationContext;
use Orbipay\PaymentsApi\Client\Model\Base\PathParamData;
use Orbipay\PaymentsApi\Client\Model\Customer\Customer;
use Orbipay\PaymentsApi\Client\Model\CustomerAccount\CustomerAccount;
use Orbipay\PaymentsApi\Client\Model\FundingAccount\FundingAccount;
use Orbipay\PaymentsApi\Client\Model\Payment\Fee;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Handler\PaymentSetup\CreatePaymentSetupHandler;
use Orbipay\PaymentsApi\Client\Handler\PaymentSetup\UpdatePaymentSetupHandler;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Model\Payment\PaymentNetworkResponse;

class PaymentSetup extends Base
{

    /**
     * @var string
     */
    private $comments;

    /**
     * @var string
     */
    private $payment_setup_reference;

    /**
     * @var string
     */
    private $currency_code_3d;

    /**
     * @var string
     */
    private $amount;

    /**
     * @var string
     */
    private $next_payment_date;

    /**
     * @var string
     */
    private $card_cvv_number;

    /**
     * @var PaymentSchedule
     */
    private $payment_schedule;

    /**
     * @var object
     */
    private $custom_fields;

    /**
     * @var Customer
     */
    private $customer;

    /**
     * @var FundingAccount
     */
    private $funding_account;

    /**
     * @var CustomerAccount
     */
    private $customer_account;

    /**
     * @var string
     */
    private $url;

    /**
     * @var string
     */
    private $id;

    /**
     * @var PaymentSetupStatus
     */
    private $status;

    /**
     * @var string
     */
    private $payment_setup_entry_date;

    /**
     * @var PaymentMethod
     */
    private $payment_method;

    /**
     * @var string
     */
    private $deleted;

    /**
     * @var string
     */
    private $confirmation_number;

    /**
     * @var Fee
     */
    private $fee;

    /**
     * @var PaymentSetupScheduleType
     */
    private $payment_setup_schedule_type;

    /**
     * @var PaymentNetworkResponse
     */
    private $payment_network_response;

    /**
     * @var AuditInfo
     */
    private $auditInfo;

    /**
     * @return AuditInfo
     */
    public function getAuditInfo()
    {
        return $this->auditInfo;
    }

    /**
     * @param AuditInfo $auditInfo
     */
    public function setAuditInfo(AuditInfo $auditInfo = null)
    {
        $this->auditInfo = $auditInfo;
    }

    /**
     * @return PaymentNetworkResponse
     */
    public function getPaymentNetworkResponse()
    {
        return $this->payment_network_response;
    }

    /**
     * @param PaymentNetworkResponse $payment_network_response
     */
    public function setPaymentNetworkResponse($payment_network_response)
    {
        $this->payment_network_response = $payment_network_response;
    }

    /**
     * PaymentSetup constructor.
     * @param string $ID_PAYMENT_SETUP
     */
    public function __construct($ID_PAYMENT_SETUP = null)
    {
        if (!($this->getPathParamData() instanceof PathParamData)) {
            $this->setPathParamData(new PathParamData());
        }
        $this->getPathParamData()->setIdPaymentSetup($ID_PAYMENT_SETUP);
        $this->id = $ID_PAYMENT_SETUP;
    }

    /**
     * @param string $amount
     * @param string $card_cvv_number
     * @param PaymentSetupScheduleType $payment_setup_schedule_type
     * @return $this
     */
    public function withDetails($amount = null, $card_cvv_number = null, PaymentSetupScheduleType $payment_setup_schedule_type = null)
    {
        $this->amount = $amount;
        $this->card_cvv_number = $card_cvv_number;
        $this->payment_setup_schedule_type = $payment_setup_schedule_type;
        return $this;
    }

    /**
     * @param string $ID_CUSTOMER
     * @return $this
     */
    public function forCustomer($ID_CUSTOMER = null)
    {
        if ($this->getPathParamData() === null) {
            $this->setPathParamData(new PathParamData());
        }
        $this->getPathParamData()->setIdCustomer($ID_CUSTOMER);
        $this->customer = new Customer($ID_CUSTOMER);
        return $this;
    }


    /**
     * @param Fee $fee
     * @return $this
     */
    public function withFee(Fee $fee = null)
    {
        $this->fee = $fee;
        return $this;
    }

    /**
     * @param PaymentSetupStatus $status
     * @return $this
     */
    public function withStatus(PaymentSetupStatus $status = null)
    {
        $this->status = $status;
        return $this;
    }

    /**
     * @param array|object $custom_fields
     * @return $this
     */
    public function withCustomFields($custom_fields = null)
    {
        if(ValidationUtil::isAssociativeArray($custom_fields)){
            $custom_fields = (object) $custom_fields;
        }
        $this->custom_fields = $custom_fields;
        return $this;
    }

    /**
     * @param string $payment_setup_reference
     * @return $this
     */
    public function withReference($payment_setup_reference = null)
    {
        $this->payment_setup_reference = $payment_setup_reference;
        return $this;
    }

    /**
     * @param string $comments
     * @return $this
     */
    public function withMemo($comments = null)
    {
        $this->comments = $comments;
        return $this;
    }

    /**
     * @param PaymentSchedule $payment_schedule
     * @return $this
     */
    public function withPaymentSchedule(PaymentSchedule $payment_schedule = null)
    {
        $this->payment_schedule = $payment_schedule;
        return $this;
    }

    /**
     * @param FundingAccount $funding_account
     * @return $this
     */
    public function fromAccount(FundingAccount $funding_account = null)
    {
        $this->funding_account = $funding_account;
        return $this;
    }

    /**
     * @param CustomerAccount $customer_account
     * @return $this
     */
    public function toAccount(CustomerAccount $customer_account = null)
    {
        $this->customer_account = $customer_account;
        return $this;
    }

    /**
     * @param string $client_key
     * @return $this
     */
    public function forClient($client_key = null)
    {
        parent::forClient($client_key);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function createdBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function updatedBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function retrievedBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function deletedBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $channel
     * @return $this
     */
    public function videChannel($channel = null)
    {
        parent::videChannel($channel);
        return $this;
    }

    /**
     * @internal
     * @param string $currency_code_3d
     */
    public function setCurrencyCode3d($currency_code_3d = null)
    {
        $this->currency_code_3d = $currency_code_3d;
    }

    /**
     * @internal
     * @param Customer $customer
     */
    public function setCustomer(Customer $customer = null)
    {
        $this->customer = $customer;
    }

    /**
     * @internal
     * @param string $next_payment_date
     */
    public function setNextPaymentDate($next_payment_date = null)
    {
        $this->next_payment_date = $next_payment_date;
    }

    /**
     * @internal
     * @param string $url
     */
    public function setUrl($url = null)
    {
        $this->url = $url;
    }

    /**
     * @internal
     * @param string $payment_setup_entry_date
     */
    public function setPaymentSetupEntryDate($payment_setup_entry_date = null)
    {
        $this->payment_setup_entry_date = $payment_setup_entry_date;
    }

    /**
     * @internal
     * @param PaymentMethod $payment_method
     */
    public function setPaymentMethod(PaymentMethod $payment_method = null)
    {
        $this->payment_method = $payment_method;
    }

    /**
     * @internal
     * @param string $deleted
     */
    public function setDeleted($deleted = null)
    {
        $this->deleted = $deleted;
    }

    /**
     * @internal
     * @param string $confirmation_number
     */
    public function setConfirmationNumber($confirmation_number = null)
    {
        $this->confirmation_number = $confirmation_number;
    }

    /**
     * @internal
     * @param Fee $fee
     */
    public function setFee(Fee $fee = null)
    {
        $this->fee = $fee;
    }

    /**
     * @return string
     */
    public function getComments()
    {
        return $this->comments;
    }

    /**
     * @return string
     */
    public function getPaymentSetupReference()
    {
        return $this->payment_setup_reference;
    }

    /**
     * @return string
     */
    public function getCurrencyCode3d()
    {
        return $this->currency_code_3d;
    }

    /**
     * @return string
     */
    public function getAmount()
    {
        return $this->amount;
    }

    /**
     * @return string
     */
    public function getNextPaymentDate()
    {
        return $this->next_payment_date;
    }

    /**
     * @return string
     */
    public function getCardCvvNumber()
    {
        return $this->card_cvv_number;
    }

    /**
     * @return PaymentSchedule
     */
    public function getPaymentSchedule()
    {
        return $this->payment_schedule;
    }

    /**
     * @return object
     */
    public function getCustomFields()
    {
        return $this->custom_fields;
    }

    /**
     * @return Customer
     */
    public function getCustomer()
    {
        return $this->customer;
    }

    /**
     * @return FundingAccount
     */
    public function getFundingAccount()
    {
        return $this->funding_account;
    }

    /**
     * @return CustomerAccount
     */
    public function getCustomerAccount()
    {
        return $this->customer_account;
    }

    /**
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * @return string
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return PaymentSetupStatus
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @return string
     */
    public function getPaymentSetupEntryDate()
    {
        return $this->payment_setup_entry_date;
    }

    /**
     * @return PaymentMethod
     */
    public function getPaymentMethod()
    {
        return $this->payment_method;
    }

    /**
     * @return string
     */
    public function getDeleted()
    {
        return $this->deleted;
    }

    /**
     * @return string
     */
    public function getConfirmationNumber()
    {
        return $this->confirmation_number;
    }

    /**
     * @return Fee
     */
    public function getFee()
    {
        return $this->fee;
    }

    /**
     * @return PaymentSetupScheduleType
     */
    public function getPaymentSetupScheduleType()
    {
        return $this->payment_setup_schedule_type;
    }

    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return PaymentSetup
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function create(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new CreatePaymentSetupHandler();
        return $handler->process($this);
    }

    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return PaymentSetup
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function get(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new GetPaymentSetupHandler();
        return $handler->process($this);
    }


    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return PaymentSetup
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function delete(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new DeletePaymentSetupHandler();
        return $handler->process($this);
    }


    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return PaymentSetup
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function update(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new UpdatePaymentSetupHandler();
        return $handler->process($this);
    }

    /**
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        return CommonUtil::json_serialize(get_object_vars($this), parent::jsonSerialize());
    }
}