<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 8/21/18
 * Time: 3:46 PM
 */

namespace Orbipay\PaymentsApi\Client\Model\Refund;


use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Enum\InvocationMode;
use Orbipay\PaymentsApi\Client\Enum\RefundStatus;
use Orbipay\PaymentsApi\Client\Enum\RefundType;
use Orbipay\PaymentsApi\Client\Enum\RequestorType;
use Orbipay\PaymentsApi\Client\Model\Base\AuditInfo;
use Orbipay\PaymentsApi\Client\Model\Base\Base;
use Orbipay\PaymentsApi\Client\Model\Base\InvocationContext;
use Orbipay\PaymentsApi\Client\Model\Base\PathParamData;
use Orbipay\PaymentsApi\Client\Model\Refund\FeeRefund;
use Orbipay\PaymentsApi\Client\Model\Payment\Payment;
use Orbipay\PaymentsApi\Client\Util\CommonUtil;
use Orbipay\PaymentsApi\Client\Handler\Refund\CreateRefundHandler;
use Orbipay\PaymentsApi\Client\Handler\Refund\GetRefundHandler;
use Orbipay\PaymentsApi\Client\Util\ValidationUtil;
use Orbipay\PaymentsApi\Client\Model\Payment\PaymentNetworkResponse;

class Refund extends Base
{
    /**
     * @var string
     */
    private $comments;
    /**
     * @var string
     */
    private $refund_reference;

    /**
     * @var RefundType
     */
    private $refund_type;

    /**
     * @var string
     */
    private $refund_amount;

    /**
     * @var string
     */
    private $id;

    /**
     * @var string
     */
    private $expected_refund_settlement_date;

    /**
     * @var string
     */
    private $refund_date;

    /**
     * @var string
     */
    private $refund_entry_date;

    /**
     * @var RefundStatus
     */
    private $status;

    /**
     * @var string
     */
    private $confirmation_number;

    /**
     * @var string
     */
    private $url;

    /**
     * @var Payment
     */
    private $source_payment;

    /**
     * @var \Orbipay\PaymentsApi\Client\Model\Refund\FeeRefund
     */
    private $fee_refund;

    /**
     * @var object
     */
    private $custom_fields;

    /**
     * @var PaymentNetworkResponse
     */
    private $payment_network_response;

    /**
     * @return PaymentNetworkResponse
     */
    public function getPaymentNetworkResponse()
    {
        return $this->payment_network_response;
    }

    /**
     * @var AuditInfo
     */
    private $auditInfo;

    /**
     * @return AuditInfo
     */
    public function getAuditInfo()
    {
        return $this->auditInfo;
    }

    /**
     * @param AuditInfo $auditInfo
     */
    public function setAuditInfo(AuditInfo $auditInfo = null)
    {
        $this->auditInfo = $auditInfo;
    }

    /**
     * @param PaymentNetworkResponse $payment_network_response
     */
    public function setPaymentNetworkResponse($payment_network_response)
    {
        $this->payment_network_response = $payment_network_response;
    }

    /**
     * Refund constructor.
     * @param string $ID_REFUND
     */
    public function __construct($ID_REFUND = null)
    {
        if (!($this->getPathParamData() instanceof PathParamData)) {
            $this->setPathParamData(new PathParamData());
        }
        $this->getPathParamData()->setIdRefund($ID_REFUND);
        $this->id = $ID_REFUND;
    }

    /**
     * @param RefundType $refund_type
     * @param string $refund_amount
     * @return $this
     */
    public function withDetails($refund_amount = null)
    {
        $this->refund_amount = $refund_amount;
        return $this;
    }

    public function withFeeRefund(FeeRefund $feeRefund = null)
    {
        $this->fee_refund = $feeRefund;
        return $this;
    }

    /**
     * @param string $refund_reference
     * @return $this
     */
    public function withReference($refund_reference = null)
    {
        $this->refund_reference = $refund_reference;
        return $this;
    }

    /**
     * @param string $comments
     * @return $this
     */
    public function withMemo($comments = null)
    {
        $this->comments = $comments;
        return $this;
    }

    /**
     * @param object $custom_fields
     * @return $this
     */
    public function withCustomFields($custom_fields = null)
    {
        if(ValidationUtil::isAssociativeArray($custom_fields)){
            $custom_fields = (object) $custom_fields;
        }
        $this->custom_fields = $custom_fields;
        return $this;
    }

    /**
     * @param string $ID_PAYMENT
     * @return $this
     */
    public function forPayment($ID_PAYMENT = null)
    {
        if ($this->getPathParamData() === null) {
            $this->setPathParamData(new PathParamData());
        }
        $this->getPathParamData()->setIdPayment($ID_PAYMENT);
        return $this;
    }

    /**
     * @param string $ID_CUSTOMER
     * @return $this
     */
    public function forCustomer($ID_CUSTOMER = null)
    {
        if ($this->getPathParamData() === null) {
            $this->setPathParamData(new PathParamData());
        }
        $this->getPathParamData()->setIdCustomer($ID_CUSTOMER);
        return $this;
    }

    /**
     * @param string $client_key
     * @return $this
     */
    public function forClient($client_key = null)
    {
        parent::forClient($client_key);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function createdBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $requestor
     * @param RequestorType $requestor_type
     * @return $this
     */
    public function retrievedBy($requestor = null, RequestorType $requestor_type = null)
    {
        $this->requestedBy($requestor, $requestor_type);
        return $this;
    }

    /**
     * @param string $channel
     * @return $this
     */
    public function videChannel($channel = null)
    {
        parent::videChannel($channel);
        return $this;
    }

    /**
     * @internal
     * @param string $expected_refund_settlement_date
     */
    public function setExpectedRefundSettlementDate($expected_refund_settlement_date = null)
    {
        $this->expected_refund_settlement_date = $expected_refund_settlement_date;
    }

    /**
     * @internal
     * @param string $refund_date
     */
    public function setRefundDate($refund_date = null)
    {
        $this->refund_date = $refund_date;
    }

    /**
     * @internal
     * @param string $refund_entry_date
     */
    public function setRefundEntryDate($refund_entry_date = null)
    {
        $this->refund_entry_date = $refund_entry_date;
    }

    /**
     * @internal
     * @param RefundStatus $status
     * @return $this
     */
    public function withStatus(RefundStatus $status = null)
    {
        $this->status = $status;
        return $this;
    }

    /**
     * @internal
     * @param string $confirmation_number
     */
    public function setConfirmationNumber($confirmation_number = null)
    {
        $this->confirmation_number = $confirmation_number;
    }

    /**
     * @internal
     * @param string $url
     */
    public function setUrl($url = null)
    {
        $this->url = $url;
    }

    /**
     * @internal
     * @param Payment $source_payment
     */
    public function setSourcePayment(Payment $source_payment = null)
    {
        $this->source_payment = $source_payment;
    }

    /**
     * @return string
     */
    public function getComments()
    {
        return $this->comments;
    }

    /**
     * @return string
     */
    public function getRefundReference()
    {
        return $this->refund_reference;
    }

    /**
     * @return RefundType
     */
    public function getRefundType()
    {
        return $this->refund_type;
    }

    /**
     * @return string
     */
    public function getRefundAmount()
    {
        return $this->refund_amount;
    }

    /**
     * @return string
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return string
     */
    public function getExpectedRefundSettlementDate()
    {
        return $this->expected_refund_settlement_date;
    }

    /**
     * @return string
     */
    public function getRefundDate()
    {
        return $this->refund_date;
    }

    /**
     * @return string
     */
    public function getRefundEntryDate()
    {
        return $this->refund_entry_date;
    }

    /**
     * @return RefundStatus
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @return string
     */
    public function getConfirmationNumber()
    {
        return $this->confirmation_number;
    }

    /**
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * @return Payment
     */
    public function getSourcePayment()
    {
        return $this->source_payment;
    }

    /**
     * @return \Orbipay\PaymentsApi\Client\Model\Refund\FeeRefund
     */
    public function getFeeRefund()
    {
        return $this->fee_refund;
    }

    /**
     * @internal
     * @param \Orbipay\PaymentsApi\Client\Model\Refund\FeeRefund $fee_refund
     */
    public function setFeeRefund(FeeRefund $fee_refund)
    {
        $this->fee_refund = $fee_refund;
    }

    /**
     *
     */
    public function getCustomFields()
    {
        return $this->custom_fields;
    }

    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return Refund
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function create(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new CreateRefundHandler();
        return $handler->process($this);
    }


    /**
     * @param InvocationContext $invocationContext
     * @param InvocationMode $invocationMode
     * @param string $apiEndPoint
     * @return Refund
     * @throws \Orbipay\PaymentsApi\Client\Exception\OrbipayApiException
     */
    public function get(InvocationContext $invocationContext, InvocationMode $invocationMode = null, $apiEndPoint = null)
    {
        $this->withContext($invocationContext, $invocationMode, $apiEndPoint);
        $handler = new GetRefundHandler();
        return $handler->process($this);
    }



    /**
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        return CommonUtil::json_serialize(get_object_vars($this), parent::jsonSerialize());
    }
}
