<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/25/18
 * Time: 5:08 PM
 */

namespace Orbipay\PaymentsApi\Client\Util;


use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;


class AuthenticationUtil
{
    const ENCODE_TYPE = 'utf-8';
    const SEPARATOR = ':';
    const KEY_VALUE_SEPARATOR = '=';
    const PARAMETER_SEPARATOR = '&';


    /**
     * @param string $method
     * @param string $resourcePath
     * @param array $headerParams
     * @param array $queryParams
     * @param array $formParams
     * @param string $httpBody
     * @param string $clientSignatureKey
     * @return array
     * @throws ApiException
     */
    public static function computeHMACSHA256Hash($method, $resourcePath, $headerParams, $queryParams,
                                                 $formParams, $httpBody, $clientSignatureKey)
    {
        $logger = LoggerFactory::getLogger(__CLASS__);

        $hash = null;
        $masked_authentication_data = '';
        try {
            /** @var string $authenticationString */
            $authenticationString = '';

            $authenticationString .= $method . self::SEPARATOR;

            $authenticationString .= RestConstants::BASE_PATH . '' . $resourcePath . self::SEPARATOR;

            $authenticationString .= self::getQueryString($queryParams) . self::SEPARATOR;

            $authenticationString .= self::getHeaderString($headerParams) . self::SEPARATOR;

            $masked_authentication_data .= $authenticationString;

            if (isset($httpBody)) {
                $authenticationString .= $httpBody;
                $masked_authentication_data .= CommonUtil::get_masked_string(json_decode($httpBody));
            } else {
                $authenticationString .= self::getFormString($formParams);
                $masked_authentication_data .= self::getFormString(CommonUtil::mask_properties($formParams));
            }

            $hash = self::generateHash($authenticationString, $clientSignatureKey);

            $logger->info('MASKED_INPUT_STRING: ' . $masked_authentication_data);
            $logger->info('HASH: ' . $hash);


        } catch (\Exception $e) {
            throw new ApiException('Unable to generate hash.');
        }
        return [$hash, $masked_authentication_data];
    }

    public static function getQueryString(array $inparams = null)
    {
        if (!is_array($inparams)) {
            return '';
        }

        $params = array();

        foreach ($inparams as $k => $v) {
            $params[trim($k)] = $v;
        }


        ksort($params);

        $queryString = '';
        foreach ($params as $k => $v) {

            if (strlen($k) == 0) {
                continue;
            }

            if (!is_array($v)) {
                if (strlen(trim($v)) > 0) {
                    $queryString .= $k . self::KEY_VALUE_SEPARATOR . trim($v) . self::PARAMETER_SEPARATOR;
                }
            } else {

                $trimmedValues = [];
                foreach ($v as $value) {
                    $trimmedValues[] = trim($value);
                }

                sort($trimmedValues);

                foreach ($trimmedValues as $value) {
                    if (strlen($value) > 0) {
                        $queryString .= $k . self::KEY_VALUE_SEPARATOR . $value . self::PARAMETER_SEPARATOR;
                    }
                }
            }
        }

        return $queryString ? (string)substr($queryString, 0, -1) : '';
    }

    public static function getFormString(array $params = null)
    {
        if (!is_array($params)) {
            return '';
        }

        $formString = '';
        foreach ($params as $k => $v) {
            if (!is_array($v)) {
                $formString .= $k . self::KEY_VALUE_SEPARATOR . $v . self::PARAMETER_SEPARATOR;
            } else {
                foreach ($v as $value) {
                    $formString .= $k . self::KEY_VALUE_SEPARATOR . $value . self::PARAMETER_SEPARATOR;
                }
            }
        }

        return $formString ? (string)substr($formString, 0, -1) : '';
    }


    public static function getHeaderString(array $inputParameters = null)
    {
        if (!is_array($inputParameters)) {
            return '';
        }

        $params = array();

        foreach ($inputParameters as $k => $v) {
            $params[trim($k)] = $v;
        }

        ksort($params);

        $headerString = '';
        foreach ($params as $k => $v) {

            if (!ValidationUtil::isValidHeaderParam($k)) {
                continue;
            }

            if (strlen(trim($v)) > 0) {
                $headerString .= $k . self::KEY_VALUE_SEPARATOR . trim($v) . self::PARAMETER_SEPARATOR;
            }

        }

        return $headerString ? (string)substr($headerString, 0, -1) : '';
    }

    /**
     * @param $data
     * @param $key
     * @return string
     */
    public static function generateHash($data, $key)
    {
        return base64_encode(hash_hmac(RestConstants::HASHING_ALGORITHM, $data, $key, true));
    }
}