<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/31/18
 * Time: 6:49 PM
 */

namespace Orbipay\PaymentsApi\Client\Util;

use Orbipay\PaymentsApi\Client\Swagger\ApiException;

class CommonUtil
{

    public static function isAPICallSuccessful(ApiException $e)
    {
        return ($e->getCode() === 200 || $e->getCode() === 201);
    }

    public static function getValidationMethod($className, $methodName)
    {
        return $className . '::' . $methodName;

    }

    public static function json_serialize($vars, $parent_vars = [])
    {
        foreach ($vars as &$value) {
            if (is_object($value) && method_exists($value, 'jsonSerialize')) {
                $value = $value->jsonSerialize();
            }
        }
        return array_merge($parent_vars, $vars);
    }

    /**
     * @param mixed $data
     * @return string
     */
    public static function get_masked_string($data)
    {
        return json_encode(self::mask_properties($data));
    }


    /**
     * @param mixed $data
     * @return mixed
     */
    public static function mask_properties($data)
    {
        if (is_array($data)) {
            return self::mask_array_properties($data);
        } else if (is_object($data)) {
            return self::mask_object_properties($data);
        } else {
            return $data;
        }
    }

    /**
     * @param $object
     * @return mixed
     */
    public static function mask_object_properties($object)
    {
        if (method_exists($object, 'jsonSerialize')) {
            return self::mask_array_properties($object->jsonSerialize());
        }

        $public = new \stdClass();
        if ($object INSTANCEOF \stdClass) {
            foreach ($object as $key => $value) {
                if (is_array($value)) {
                    $public->$key = self::mask_array_properties($value);
                } else if (is_object($value)) {
                    $public->$key = self::mask_object_properties($value);
                } else {
                    $sensitive_info = self::getSensitiveInfo();
                    if (array_key_exists($key, $sensitive_info)) {
                        $public->$key = self::mask_string($value, $sensitive_info[$key]);
                    } else {
                        $public->$key = $value;
                    }
                }
            }
        }
        return $public;
    }

    /**
     * @param $array
     * @return array
     */
    public static function mask_array_properties($array)
    {
        $public = [];
        if (ValidationUtil::isAssociativeArray($array)) {
            foreach ($array as $key => $value) {
                if (is_array($value)) {
                    $public[$key] = self::mask_array_properties($value);
                } else if (is_object($value)) {
                    $public[$key] = self::mask_object_properties($value);
                } else {
                    $sensitive_info = self::getSensitiveInfo();
                    if (array_key_exists($key, $sensitive_info)) {
                        $public[$key] = self::mask_string($value, $sensitive_info[$key]);
                    } else {
                        $public[$key] = $value;
                    }
                }
            }
        } else {
            foreach ($array as $item) {
                if (is_object($item)) {
                    $public[] = self::mask_object_properties($item);
                } else if (is_array($item)) {
                    $public[] = self::mask_array_properties($item);
                } else {
                    $public[] = $item;
                }
            }
        }
        return $public;
    }

    public static function mask_string($data, $visible_length = 0)
    {
        if (!is_string($data) || strlen($data) === 0) {
            return $data;
        }

        $string_length = strlen($data);

        if (!is_int($visible_length) || $visible_length <= 0 || ($string_length - $visible_length) <= 0) {
            return str_repeat('X', $string_length);
        }

        return str_repeat('X', ($string_length - $visible_length)) . substr($data, -$visible_length);
    }

    public static function getSensitiveInfo()
    {
        return ['account_number' => 3, 'ssn' => 0, 'card_cvv_number' => 0, 'secret' => 0];
    }

    public static function isJson($string)
    {
        json_decode($string);
        return (json_last_error() == JSON_ERROR_NONE);
    }
}