<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/21/18
 * Time: 5:05 PM
 */

namespace Orbipay\PaymentsApi\Client\Util;

use Orbipay\PaymentsApi\Client\Model\Base\Address;
use Orbipay\PaymentsApi\Client\Model\Base\Attribute;
use Orbipay\PaymentsApi\Client\Model\Base\Attributes;
use Orbipay\PaymentsApi\Client\Model\Customer\Customer;
use Orbipay\PaymentsApi\Client\Model\CustomerAccount\CustomerAccount;
use Orbipay\PaymentsApi\Client\Model\FundingAccount\FundingAccount;
use Orbipay\PaymentsApi\Client\Model\Payment\Fee;
use Orbipay\PaymentsApi\Client\Model\Refund\FeeRefund;
use Orbipay\PaymentsApi\Client\Swagger\Model\CreateCustomerAccountRequest;
use Orbipay\PaymentsApi\Client\Swagger\Model\SimplePaymentCustomerRequest;
use Orbipay\PaymentsApi\Client\Swagger\Model\SimplePaymentFundingAccountRequest;
use Psr\Http\Message\RequestInterface;
use Orbipay\PaymentsApi\Client\Swagger\Model\CustomerAccountMinimalRequest;
use Orbipay\PaymentsApi\Client\Swagger\Model\CustomerMinimalRequest;
use Orbipay\PaymentsApi\Client\Swagger\Model\FeeRequest;
use Orbipay\PaymentsApi\Client\Swagger\Model\FundingAccountMinimalRequest;

class RequestWrapper
{
    /**
     * @param $address
     * @return \Orbipay\PaymentsApi\Client\Swagger\Model\AddressVO
     */
    public static final function wrapAddress($address)
    {
        $addressRequest = null;
        if ($address instanceof Address) {
            $addressRequest = new \Orbipay\PaymentsApi\Client\Swagger\Model\AddressVO();
            $addressRequest->setAddressLine1($address->getAddressLine1());
            $addressRequest->setAddressLine2($address->getAddressLine2());
            $addressRequest->setAddressCity($address->getAddressCity());
            $addressRequest->setAddressState($address->getAddressState());
            $addressRequest->setAddressCountry($address->getAddressCountry());
            $addressRequest->setAddressZip1($address->getAddressZip1());
            $addressRequest->setAddressZip2($address->getAddressZip2());
        }
        return $addressRequest;
    }

    /**
     * @param $attributes
     * @return string
     */
    public static function wrapXOPAYHeaders($attributes)
    {
        $xOPAYHeaders = '';
        if ($attributes instanceof Attributes && is_array($attributes->getList())) {
            $list = $attributes->getList();
            for ($i = 0; $i < count($list); $i++) {
                $attribute = $list[$i];
                if ($attribute instanceof Attribute) {
                    $key = $attribute->getKey();
                    $value = $attribute->getValue();
                    if (ValidationUtil::isNonEmptyString($key) && ValidationUtil::isNonEmptyString($value)) {
                        $xOPAYHeaders .= $key . '=' . $value . ';';
                    }
                }
            }
        }
        return $xOPAYHeaders ? (string)substr($xOPAYHeaders, 0, -1) : '';
    }

    public static function wrapHttpRequest(RequestInterface $http_request = null, $_tempBody = null, $formParams = null)
    {
        $request = array();
        if ($http_request != null) {
            $request['method'] = $http_request->getMethod();
            $request['uri'] = $http_request->getUri();
            $request['headers'] = $http_request->getHeaders();
            $request['http_protocol_version'] = $http_request->getProtocolVersion();
        }

        if (isset($_tempBody)) {
            $request['body'] = json_decode($_tempBody);
        } else {
            $request['body'] = AuthenticationUtil::getFormString(CommonUtil::mask_properties($formParams));
        }

        return $request;
    }

    public static function getFeeRequest(Fee $fee = null)
    {
        $feeRequest = null;
        if (isset($fee)) {
            $feeRequest = new FeeRequest();
            $feeRequest->setFeeAmount($fee->getFeeAmount());
        }
        return $feeRequest;
    }
    public static function getFeeRefundRequest(FeeRefund $fee = null)
    {
        $feeRequest = null;
        if (isset($fee)) {
            $feeRequest = new FeeRequest();
            $feeRequest->setFeeAmount($fee->getFeeRefundAmount());
        }
        return $feeRequest;
    }
    public static function getCustomerMinimalRequest(Customer $customer = null)
    {
        $customerMinimal = null;
        if (isset($customer)) {
            $customerMinimal = new CustomerMinimalRequest();
            $customerMinimal->setId($customer->getId());
        }
        return $customerMinimal;
    }

    public static function getCustomerAccountMinimalRequest(CustomerAccount $customerAccount = null)
    {
        $customerAccountMinimal = null;
        if (isset($customerAccount)) {
            $customerAccountMinimal = new CustomerAccountMinimalRequest();
            $customerAccountMinimal->setId($customerAccount->getId());
        }
        return $customerAccountMinimal;
    }

    public static function getFundingAccountMinimalRequest(FundingAccount $fundingAccount = null)
    {
        $fundingAccountMinimal = null;
        if (isset($fundingAccount)) {
            $fundingAccountMinimal = new FundingAccountMinimalRequest();
            $fundingAccountMinimal->setId($fundingAccount->getId());
        }
        return $fundingAccountMinimal;
    }

    public static function getSimplePaymentCustomerRequest(Customer $customer = null)
    {
        $customerRequest = null;

        if($customer != null){
            $customerRequest = new SimplePaymentCustomerRequest();
            $customerRequest->setCustomerReference($customer->getCustomerReference());
            $customerRequest->setFirstName($customer->getFirstName());
            $customerRequest->setLastName($customer->getLastName());
            $customerRequest->setMiddleName($customer->getMiddleName());
            if ($customer->getGender() !== null) {
                $customerRequest->setGender($customer->getGender()->value());
            }
            $customerRequest->setDateOfBirth($customer->getDateOfbirth());
            $customerRequest->setSsn($customer->getSsn());
            $customerRequest->setLocale($customer->getLocale());
            $customerRequest->setEmail($customer->getEmail());
            $customerRequest->setRegisteredEmail($customer->getRegisteredEmail());
            $customerRequest->setHomePhone($customer->getHomePhone());
            $customerRequest->setWorkPhone($customer->getWorkPhone());
            $customerRequest->setMobilePhone($customer->getMobilePhone());
            $customerRequest->setAddress(RequestWrapper::wrapAddress($customer->getAddress()));
            $customerRequest->setCustomFields((object)$customer->getCustomFields());
        }

        return $customerRequest;
    }
    public static function getSimplePaymentCustomerAccountRequest(CustomerAccount $customerAccount = null)
    {
        $customerAccountRequest = null;
        if ($customerAccount !== null) {
            $customerAccountRequest = new CreateCustomerAccountRequest();
            $customerAccountRequest->setAccountHolderName($customerAccount->getAccountHolderName());
            $customerAccountRequest->setNickname($customerAccount->getNickname());
            $customerAccountRequest->setAddress(RequestWrapper::wrapAddress($customerAccount->getAddress()));
            $customerAccountRequest->setCustomerAccountReference($customerAccount->getCustomerAccountReference());
            $customerAccountRequest->setAccountNumber($customerAccount->getAccountNumber());
            $customerAccountRequest->setCurrentBalance($customerAccount->getCurrentBalance());
            $customerAccountRequest->setCurrentStatementBalance($customerAccount->getCurrentStatementBalance());
            $customerAccountRequest->setMinimumPaymentDue($customerAccount->getMinimumPaymentDue());
            $customerAccountRequest->setPastAmountDue($customerAccount->getPastAmountDue());
            $customerAccountRequest->setPaymentDueDate($customerAccount->getPaymentDueDate());
            $customerAccountRequest->setStatementDate($customerAccount->getStatementDate());
            $customerAccountRequest->setCustomFields((object)$customerAccount->getCustomFields());
        }

        return $customerAccountRequest;
    }

    public static function getSimplePaymentFundingAccountRequest(FundingAccount $fundingAccount = null)
    {
        $fundingAccountRequest = null;
        if($fundingAccount !=null){
            $fundingAccountRequest = new SimplePaymentFundingAccountRequest();
            $fundingAccountRequest->setAccountHolderName($fundingAccount->getAccountHolderName());
            $fundingAccountRequest->setNickname($fundingAccount->getNickname());
            $fundingAccountRequest->setAddress(RequestWrapper::wrapAddress($fundingAccount->getAddress()));
            $fundingAccountRequest->setAccountNumber($fundingAccount->getAccountNumber());
            $fundingAccountRequest->setAbaRoutingNumber($fundingAccount->getAbaRoutingNumber());
            $fundingAccountRequest->setExpiryDate($fundingAccount->getExpiryDate());
            $fundingAccountRequest->setAccountHolderType($fundingAccount->getAccountHolderType() !== null ? $fundingAccount->getAccountHolderType()->value() : null);
            $fundingAccountRequest->setCustomFields($fundingAccount->getCustomFields());
            $fundingAccountRequest->setAccountSubtype($fundingAccount->getAccountSubtype() !== null ? $fundingAccount->getAccountSubtype()->value() : null);
            $fundingAccountRequest->setCardCvvNumber($fundingAccount->getCardCvvNumber());
        }
        return $fundingAccountRequest;
    }

}