<?php
/**
 * Created by PhpStorm.
 * User: Alacriti Inc.
 * Date: 7/21/18
 * Time: 5:05 PM
 */

namespace Orbipay\PaymentsApi\Client\Util;


use Orbipay\PaymentsApi\Client\Constant\HttpParamConstants;
use Orbipay\PaymentsApi\Client\Constant\RestConstants;
use Orbipay\PaymentsApi\Client\Enum\ACHEligibleFlag;
use Orbipay\PaymentsApi\Client\Enum\ATMEligibleFlag;
use Orbipay\PaymentsApi\Client\Enum\CustomerAccountStatus;
use Orbipay\PaymentsApi\Client\Enum\CustomerStatus;
use Orbipay\PaymentsApi\Client\Enum\FeeType;
use Orbipay\PaymentsApi\Client\Enum\FundingAccountStatus;
use Orbipay\PaymentsApi\Client\Enum\FundingAccountSubtype;
use Orbipay\PaymentsApi\Client\Enum\FundingAccountType;
use Orbipay\PaymentsApi\Client\Enum\Gender;
use Orbipay\PaymentsApi\Client\Enum\PaymentAmountType;
use Orbipay\PaymentsApi\Client\Enum\PaymentMethod;
use Orbipay\PaymentsApi\Client\Enum\PaymentRecurringType;
use Orbipay\PaymentsApi\Client\Enum\PaymentScheduleType;
use Orbipay\PaymentsApi\Client\Enum\PaymentSetupScheduleType;
use Orbipay\PaymentsApi\Client\Enum\PaymentStatus;
use Orbipay\PaymentsApi\Client\Enum\PaymentSetupStatus;
use Orbipay\PaymentsApi\Client\Enum\Product;
use Orbipay\PaymentsApi\Client\Enum\RefundStatus;
use Orbipay\PaymentsApi\Client\Enum\RefundType;
use Orbipay\PaymentsApi\Client\Enum\RequestorType;
use Orbipay\PaymentsApi\Client\Model\Base\Address;
use Orbipay\PaymentsApi\Client\Model\Base\Attributes;
use Orbipay\PaymentsApi\Client\Model\Base\AuditInfo;
use Orbipay\PaymentsApi\Client\Model\Base\Base;
use Orbipay\PaymentsApi\Client\Model\Base\Created;
use Orbipay\PaymentsApi\Client\Model\Base\InvocationContext;
use Orbipay\PaymentsApi\Client\Model\Base\LastModified;
use Orbipay\PaymentsApi\Client\Model\Base\ResponseCode;
use Orbipay\PaymentsApi\Client\Model\Customer\Customer;
use Orbipay\PaymentsApi\Client\Model\Customer\Customers;
use Orbipay\PaymentsApi\Client\Model\CustomerAccount\CustomerAccount;
use Orbipay\PaymentsApi\Client\Model\CustomerAccount\CustomerAccounts;
use Orbipay\PaymentsApi\Client\Model\Error\Error;
use Orbipay\PaymentsApi\Client\Model\FeePayment\FeePayment;
use Orbipay\PaymentsApi\Client\Model\FundingAccount\FundingAccount;
use Orbipay\PaymentsApi\Client\Model\FundingAccount\FundingAccounts;
use Orbipay\PaymentsApi\Client\Model\Payment\PaymentNetworkResponse;
use Orbipay\PaymentsApi\Client\Model\Payment\SimplePayment;
use Orbipay\PaymentsApi\Client\Model\Refund\FeeRefund;
use Orbipay\PaymentsApi\Client\Model\Payment\Payment;
use Orbipay\PaymentsApi\Client\Model\Payment\Fee;
use Orbipay\PaymentsApi\Client\Model\Payment\Payments;
use Orbipay\PaymentsApi\Client\Model\PaymentSetup\PaymentSchedule;
use Orbipay\PaymentsApi\Client\Model\PaymentSetup\PaymentSetup;
use Orbipay\PaymentsApi\Client\Model\PaymentSetup\PaymentSetups;
use Orbipay\PaymentsApi\Client\Model\Refund\Refund;
use Orbipay\PaymentsApi\Client\Swagger\Model\SourcePaymentCustomer;
use Orbipay\PaymentsApi\Client\Swagger\Model\SourcePaymentCustomerAccount;
use Psr\Http\Message\ResponseInterface;
use Orbipay\PaymentsApi\Client\Swagger\ApiException;
use Orbipay\PaymentsApi\Client\Swagger\Model\AddressVO;
use Orbipay\PaymentsApi\Client\Swagger\Model\CustomerAccountsResponse;
use Orbipay\PaymentsApi\Client\Swagger\Model\CustomerMinimalResponse;
use Orbipay\PaymentsApi\Client\Swagger\Model\CustomerResponse;
use Orbipay\PaymentsApi\Client\Swagger\Model\CustomersCustomerAccountsResponse;
use Orbipay\PaymentsApi\Client\Swagger\Model\CustomerSnapshotResponse;
use Orbipay\PaymentsApi\Client\Swagger\Model\CustomersResponse;
use Orbipay\PaymentsApi\Client\Swagger\Model\DeleteResponse;
use Orbipay\PaymentsApi\Client\Swagger\Model\FundingAccountsResponse;
use Orbipay\PaymentsApi\Client\Swagger\Model\PaymentSetupsResponse;
use Orbipay\PaymentsApi\Client\Swagger\Model\PaymentsResponse;
use Orbipay\PaymentsApi\Client\Swagger\Model\RefundResponse;
use Orbipay\PaymentsApi\Client\Swagger\Model\RefundsResponse;


class ResponseWrapper
{
    public static function wrapCustomer(CustomerResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $customer = new Customer($response->getId());
        $customer->withDetails($response->getFirstName(), $response->getLastName(), $response->getMiddleName(), $response->getDateOfBirth(),
            $response->getSsn(), Gender::fromValue($response->getGender()), $response->getRegisteredEmail());
        $customer->withContactInfo($response->getEmail(), $response->getHomePhone(), $response->getWorkPhone(), $response->getMobilePhone());
        $customer->withAddress(self::wrapAddress($response->getAddress()));
        $customer->withReference($response->getCustomerReference());
        $customer->withStatus(CustomerStatus::fromValue($response->getStatus()));
        $customer->withCustomFields($response->getCustomFields());
        $customer->setUrl($response->getUrl());
        $customer->withAccounts(self::wrapCustomersCustomerAccounts($response->getCustomerAccounts()));
        $customer->setPayments(self::wrapCustomersPayments($response->getPayments()));
        $customer->setFundingAccounts(self::wrapCustomersFundingAccounts($response->getFundingAccounts()));
        $customer->setPaymentSetups(self::wrapCustomersPaymentSetups($response->getPaymentSetups()));
        $customer->setAuditInfo(self::wrapAuditInfo($response->getAuditInfo()));
        if (is_string($response->getLocale())) {
            $array = explode(RestConstants::LOCALE_DELIMITER, $response->getLocale());

            if (count($array) == 2) {
                $customer->withLocale($array[0], $array[1]);
            }
        }
        return $customer;
    }


    public static function wrapCustomerSnapshotResponse(CustomerSnapshotResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $customer = new Customer($response->getId());
        $customer->withDetails($response->getFirstName(), $response->getLastName(), $response->getMiddleName(), null,
            null, null);
        $customer->withContactInfo($response->getEmail(), $response->getHomePhone(), null, null);
        $customer->withAddress(self::wrapAddress($response->getAddress()));
        $customer->withReference(null);
        $customer->setUrl($response->getUrl());

        return $customer;
    }

    public static function wrapSourcePaymentCustomerResponse(SourcePaymentCustomer $response = null)
    {
        if ($response === null) {
            return null;
        }

        $customer = new Customer($response->getId());
        $customer->setUrl($response->getUrl());
        $customer->withReference($response->getCustomerReference());

        return $customer;
    }

    public static function wrapSourcePaymentCustomerAccountResponse(SourcePaymentCustomerAccount $response = null)
    {
        if ($response === null) {
            return null;
        }

        $customerAccount = new CustomerAccount($response->getId());
        $customerAccount->setUrl($response->getUrl());
        $customerAccount->withDetails(null, null, $response->getAccountNumber(),null,null,null, null);
        return $customerAccount;
    }

    /**
     * @param AddressVO $response
     * @return \Orbipay\PaymentsApi\Client\Model\Base\Address
     */
    public static function wrapAddress(AddressVO $response = null)
    {
        if ($response === null) {
            return null;
        }

        return new Address($response->getAddressLine1(), $response->getAddressLine2(),
            $response->getAddressCity(), $response->getAddressState(), $response->getAddressCountry(), $response->getAddressZip1(), $response->getAddressZip2());
    }

    public static function wrapCustomersCustomerAccounts(CustomersCustomerAccountsResponse $response = null)
    {
        if ($response === null || $response->getList() === null || count($response->getList()) === 0) {
            return null;
        }

        $customerAccounts = new CustomerAccounts();
        $customerAccounts->setUrl($response->getUrl());

        /** @var \Orbipay\PaymentsApi\Client\Swagger\Model\CustomerAccountResponse $accountResponse */
        foreach ($response->getList() as $accountResponse) {
            $customerAccounts->add(self::wrapCustomerAccount($accountResponse));
        }
        return $customerAccounts;
    }

    public static function wrapCustomerAccount(\Orbipay\PaymentsApi\Client\Swagger\Model\CustomerAccountResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $customerAccount = new CustomerAccount($response->getId());

        $customerAccount->setUrl($response->getUrl());
        $customerAccount->withDetails($response->getAccountHolderName(), $response->getNickname(), $response->getAccountNumber(), $response->getMinimumPaymentDue(), $response->getPastAmountDue(), $response->getPaymentDueDate(), $response->getStatementDate());
        $customerAccount->withBalances($response->getCurrentBalance(), $response->getCurrentStatementBalance());
        $customerAccount->withReference($response->getCustomerAccountReference());
        $customerAccount->withStatus(CustomerAccountStatus::fromValue($response->getStatus()));
        $customerAccount->withAddress(self::wrapAddress($response->getAddress()));
        $customerAccount->withCustomFields($response->getCustomFields());
        $customerAccount->setAuditInfo(self::wrapAuditInfo($response->getAuditInfo()));
        $customerAccount->withMemo($response->getComments());

        return $customerAccount;
    }

    public static function wrapFundingAccount(\Orbipay\PaymentsApi\Client\Swagger\Model\FundingAccountResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $fundingAccount = new FundingAccount($response->getId());

        $fundingAccount->withDetails($response->getAccountHolderName(), $response->getNickname(), $response->getAccountNumber(), FundingAccountSubtype::fromValue($response->getAccountSubtype()), $response->getAbaRoutingNumber(), $response->getCardCvvNumber(), $response->getExpiryDate(), $response->getAccountHolderType());
        $fundingAccount->withStatus(FundingAccountStatus::fromValue($response->getStatus()));
        $fundingAccount->withCustomFields($response->getCustomFields());
        $fundingAccount->withAddress(self::wrapAddress($response->getAddress()));
        $fundingAccount->setUrl($response->getUrl());
        $fundingAccount->setAtmEligibleFlag(ATMEligibleFlag::fromValue($response->getAtmEligibleFlag()));
        $fundingAccount->setIssuerName($response->getIssuerName());
        $fundingAccount->setAccountType(FundingAccountType::fromValue($response->getAccountType()));
        $fundingAccount->setAchEligibleFlag(ACHEligibleFlag::fromValue($response->getAchEligibleFlag()));
        $fundingAccount->setCurrencyCode3d($response->getCurrencyCode3d());
        $fundingAccount->setAuditInfo(self::wrapAuditInfo($response->getAuditInfo()));

        return $fundingAccount;
    }

    public static function wrapFundingAccountSnapshotResponse(\Orbipay\PaymentsApi\Client\Swagger\Model\FundingAccountSnapshotResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $fundingAccount = new FundingAccount($response->getId());

        $fundingAccount->withDetails(null, null, $response->getAccountNumber(), FundingAccountSubtype::fromValue($response->getAccountSubtype()), $response->getAbaRoutingNumber(), null, null, null);
        $fundingAccount->setUrl($response->getUrl());
        $fundingAccount->setAccountType(FundingAccountType::fromValue($response->getAccountType()));
        return $fundingAccount;
    }

    public static function wrapCustomerAccountSnapshotResponse(\Orbipay\PaymentsApi\Client\Swagger\Model\CustomerAccountSnapshotResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $customerAccount = new CustomerAccount($response->getId());

        $customerAccount->setUrl($response->getUrl());

        return $customerAccount;
    }

    public static function wrapFee(\Orbipay\PaymentsApi\Client\Swagger\Model\FeeResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $fee = new Fee($response->getFeeAmount());
        $fee->setUrl($response->getUrl());
        $fee->setFeeType(FeeType::fromValue($response->getFeeType()));
        $fee->setId($response->getId());

        return $fee;
    }

    public static function wrapPaymentNetworkResponse(\Orbipay\PaymentsApi\Client\Swagger\Model\PaymentNetworkResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $payment_network_response = new PaymentNetworkResponse();
        $payment_network_response->setPaymentAuthCode($response->getPaymentAuthCode());
        $payment_network_response->setPaymentResponseCode($response->getPaymentResponseCode());
        $payment_network_response->setPaymentResponseMsg($response->getPaymentResponseMsg());

        return $payment_network_response;
    }

    public static function wrapFeeRefund(\Orbipay\PaymentsApi\Client\Swagger\Model\FeeRefund $response = null)
    {
        if ($response === null) {
            return null;
        }

        $fee = new FeeRefund($response->getFeeRefundAmount());
        $fee->setFeeType(FeeType::fromValue($response->getFeeType()));

        return $fee;
    }

    public static function wrapDeletePaymentResponse(DeleteResponse $response = null)
    {
        $payment = null;
        if ($response != null) {
            $payment = new Payment($response->getId());
            $payment->setDeleted($response->getDeleted());
        }
        return $payment;
    }

    public static function wrapDeletePaymentSetupResponse(DeleteResponse $response = null)
    {
        $paymentSetup = null;
        if ($response != null) {
            $paymentSetup = new PaymentSetup($response->getId());
            $paymentSetup->setDeleted($response->getDeleted());
        }
        return $paymentSetup;
    }

    public static function wrapDeleteFundingAccountResponse(DeleteResponse $response = null)
    {
        $fundingAccount = null;
        if ($response != null) {
            $fundingAccount = new FundingAccount($response->getId());
            $fundingAccount->setDeleted($response->getDeleted());
        }
        return $fundingAccount;
    }

    public static function wrapPayment(\Orbipay\PaymentsApi\Client\Swagger\Model\PaymentResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $payment = new Payment($response->getId());
        $payment->withDetails(PaymentAmountType::fromValue($response->getPaymentAmountType()), $response->getAmount(), $response->getPaymentDate(), $response->getPaymentRequestDate(), $response->getCardCvvNumber());
        $payment->withStatus(PaymentStatus::fromValue($response->getStatus()));
        $payment->withReference($response->getPaymentReference());
        $payment->withCustomFields($response->getCustomFields());
        $payment->fromAccount(self::wrapFundingAccountSnapshotResponse($response->getFundingAccount()));
        $payment->toAccount(self::wrapCustomerAccountSnapshotResponse($response->getCustomerAccount()));
        $payment->setCustomer(self::wrapCustomerSnapshotResponse($response->getCustomer()));
        $payment->setUrl($response->getUrl());
        $payment->setConfirmationNumber($response->getConfirmationNumber());
        $payment->setPaymentEntryDate($response->getPaymentEntryDate());
        $payment->setPaymentReturnDate($response->getPaymentReturnDate());
        $payment->setReturnCode($response->getReturnCode());
        $payment->setPaymentMethod(PaymentMethod::fromValue($response->getPaymentMethod()));
        $payment->setPaymentScheduleType(PaymentScheduleType::fromValue($response->getPaymentScheduleType()));
        $payment->setExpectedPaymentSettlementDate($response->getExpectedPaymentSettlementDate());
        $payment->setCurrencyCode3d($response->getCurrencyCode3d());
        $payment->setFee(self::wrapFee($response->getFee()));
        $payment->setPaymentNetworkResponse(self::wrapPaymentNetworkResponse($response->getPaymentNetworkResponse()));
        $payment->setAuditInfo(self::wrapAuditInfo($response->getAuditInfo()));

        return $payment;
    }

    public static function wrapFeePayment(\Orbipay\PaymentsApi\Client\Swagger\Model\PaymentResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $payment = new FeePayment($response->getId());
        $payment->withDetails(PaymentAmountType::fromValue($response->getPaymentAmountType()), $response->getAmount(), $response->getPaymentDate(), $response->getPaymentRequestDate(), $response->getCardCvvNumber());
        $payment->withStatus(PaymentStatus::fromValue($response->getStatus()));
        $payment->withReference($response->getPaymentReference());
        $payment->withCustomFields($response->getCustomFields());
        $payment->fromAccount(self::wrapFundingAccountSnapshotResponse($response->getFundingAccount()));
        $payment->toAccount(self::wrapCustomerAccountSnapshotResponse($response->getCustomerAccount()));
        $payment->setCustomer(self::wrapCustomerSnapshotResponse($response->getCustomer()));
        $payment->setUrl($response->getUrl());
        $payment->setConfirmationNumber($response->getConfirmationNumber());
        $payment->setPaymentEntryDate($response->getPaymentEntryDate());
        $payment->setPaymentReturnDate($response->getPaymentReturnDate());
        $payment->setReturnCode($response->getReturnCode());
        $payment->setPaymentMethod(PaymentMethod::fromValue($response->getPaymentMethod()));
        $payment->setPaymentScheduleType(PaymentScheduleType::fromValue($response->getPaymentScheduleType()));
        $payment->setExpectedPaymentSettlementDate($response->getExpectedPaymentSettlementDate());
        $payment->setCurrencyCode3d($response->getCurrencyCode3d());
        $payment->setPaymentNetworkResponse(self::wrapPaymentNetworkResponse($response->getPaymentNetworkResponse()));

        return $payment;
    }


    public static function wrapPaymentSetup(\Orbipay\PaymentsApi\Client\Swagger\Model\PaymentSetupResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $paymentSetup = new PaymentSetup($response->getId());
        $paymentSetup->withDetails($response->getAmount(), $response->getCardCvvNumber(), PaymentSetupScheduleType::fromValue($response->getPaymentSetupScheduleType()));
        $paymentSetup->withReference($response->getPaymentSetupReference());
        $paymentSetup->withStatus(PaymentSetupStatus::fromValue($response->getStatus()));
        $paymentSetup->withCustomFields($response->getCustomFields());
        $paymentSetup->fromAccount(self::wrapFundingAccountSnapshotResponse($response->getFundingAccount()));
        $paymentSetup->toAccount(self::wrapCustomerAccountSnapshotResponse($response->getCustomerAccount()));
        $paymentSetup->setCustomer(self::wrapCustomerSnapshotResponse($response->getCustomer()));
        $paymentSetup->setUrl($response->getUrl());
        $paymentSetup->setConfirmationNumber($response->getConfirmationNumber());
        $paymentSetup->setNextPaymentDate($response->getNextPaymentDate());
        $paymentSetup->setPaymentSetupEntryDate($response->getPaymentSetupEntryDate());
        $paymentSetup->setPaymentMethod(PaymentMethod::fromValue($response->getPaymentMethod()));
        $paymentSetup->setCurrencyCode3d($response->getCurrencyCode3d());
        $paymentSetup->setFee(self::wrapFee($response->getFee()));
        $paymentSetup->withPaymentSchedule(self::wrapPaymentSchedule($response->getPaymentSchedule()));
        $paymentSetup->setPaymentNetworkResponse(self::wrapPaymentNetworkResponse($response->getPaymentNetworkResponse()));
        $paymentSetup->setAuditInfo(self::wrapAuditInfo($response->getAuditInfo()));

        return $paymentSetup;
    }

    public static function wrapPaymentSchedule(\Orbipay\PaymentsApi\Client\Swagger\Model\PaymentScheduleVO $response = null)
    {
        if ($response === null) {
            return null;
        }

        $paymentSchedule = new PaymentSchedule(PaymentRecurringType::fromValue($response->getPaymentRecurringType()), $response->getPaymentRecurringCount(), PaymentAmountType::fromValue($response->getPaymentAmountType()), $response->getPaymentStartDate(), $response->getPaymentEndDate(), $response->getPaymentLimitAmount(), $response->getPaymentPlanId(), $response->getPaymentDeferDays());

        return $paymentSchedule;
    }

    /**
     * @param string $xOPAYHeaders
     * @return Attributes
     *
     */
    public static function getAttributes($xOPAYHeaders = null)
    {
        $attributes = null;

        if (!ValidationUtil::isNonEmptyString($xOPAYHeaders)) {
            return $attributes;
        }

        $xOPAYHeadersArray = explode(';', $xOPAYHeaders);
        $attributes = new Attributes();

        foreach ($xOPAYHeadersArray as $header) {
            if (!ValidationUtil::isNonEmptyString($header)) {
                continue;
            }
            $index = strpos($header, '=');
            if ($index > 0) {
                $key = substr($header, 0, $index);
                if ($index + 1 < strlen($header)) {
                    $value = substr($header, $index + 1, strlen($header));
                    $attributes->add($key, $value);
                }
            }
        }

        return $attributes;
    }

    /**
     * @param ResponseInterface $response
     * @return array
     */
    public static function getResponseCodes(ResponseInterface $response = null)
    {
        $responseCodes = null;

        if ($response !== null && count($response->getHeader(HttpParamConstants::RESPONSE_CODES)) > 0 && count($response->getHeader(HttpParamConstants::RESPONSE_TEXT)) > 0) {

            $responseCodesHeader = explode(',', array_values($response->getHeader(HttpParamConstants::RESPONSE_CODES))[0]);
            $responseTextHeader = explode('|', array_values($response->getHeader(HttpParamConstants::RESPONSE_TEXT))[0]);

            if (is_array($responseCodesHeader) && is_array($responseTextHeader) && count($responseCodesHeader) === count($responseTextHeader)) {
                $responseCodes = array();
                for ($i = 0; $i < count($responseCodesHeader); $i++) {
                    $responseCodes[] = new ResponseCode($responseCodesHeader[$i], $responseTextHeader[$i]);
                }
            }
        }

        return $responseCodes;
    }

    /**
     * @param array $response
     * @return InvocationContext
     *
     */
    public static function getInvocationContext(array $headers = null, Base $object = null)
    {
        /**
         * @var InvocationContext
         */
        $invocationContext = null;

        if ($headers !== null) {

            $idempotentRequestKey = null;
            if (isset($headers[HttpParamConstants::IDEMPOTENT_REQUEST_KEY]) && count($headers[HttpParamConstants::IDEMPOTENT_REQUEST_KEY]) > 0) {
                $idempotentRequestKey = array_values($headers[HttpParamConstants::IDEMPOTENT_REQUEST_KEY])[0];
            }

            $attributes = null;
            if (isset($headers[HttpParamConstants::X_OPAY_HEADERS]) && count($headers[HttpParamConstants::X_OPAY_HEADERS]) > 0) {
                $attributes = self::getAttributes(array_values($headers[HttpParamConstants::X_OPAY_HEADERS])[0]);
            }

            $invocationContext = new InvocationContext($idempotentRequestKey, null, $attributes);

            if (isset($headers[HttpParamConstants::CHANNEL]) && count($headers[HttpParamConstants::CHANNEL]) > 0) {
                $invocationContext->setChannel(array_values($headers[HttpParamConstants::CHANNEL])[0]);
            }

            if (isset($headers[HttpParamConstants::CLIENT_KEY]) && count($headers[HttpParamConstants::CLIENT_KEY]) > 0) {
                $invocationContext->setClientKey(array_values($headers[HttpParamConstants::CLIENT_KEY])[0]);
            }

            if (isset($headers[HttpParamConstants::PRODUCT]) && count($headers[HttpParamConstants::PRODUCT]) > 0) {
                $invocationContext->setProduct(Product::fromValue(array_values($headers[HttpParamConstants::PRODUCT])[0]));
            }

            if (isset($headers[HttpParamConstants::TIMESTAMP]) && count($headers[HttpParamConstants::TIMESTAMP]) > 0) {
                $invocationContext->setTimestamp(array_values($headers[HttpParamConstants::TIMESTAMP])[0]);
            }

            if (isset($headers[HttpParamConstants::REQUESTOR]) && count($headers[HttpParamConstants::REQUESTOR]) > 0) {
                $invocationContext->setRequestor(array_values($headers[HttpParamConstants::REQUESTOR])[0]);
            }

            if (isset($headers[HttpParamConstants::REQUESTOR_TYPE]) && count($headers[HttpParamConstants::REQUESTOR_TYPE]) > 0) {
                $invocationContext->setRequestorType(RequestorType::fromValue(array_values($headers[HttpParamConstants::REQUESTOR_TYPE])[0]));
            }

            if (isset($headers[HttpParamConstants::REQUEST_UUID]) && count($headers[HttpParamConstants::REQUEST_UUID]) > 0) {
                $invocationContext->setRequestUuid(array_values($headers[HttpParamConstants::REQUEST_UUID])[0]);
            }

            if (isset($headers[HttpParamConstants::RESPONSE_CODES]) && count($headers[HttpParamConstants::RESPONSE_CODES]) > 0) {
                $invocationContext->setResponseCodes(array_values($headers[HttpParamConstants::RESPONSE_CODES])[0]);
            }

            if (isset($headers[HttpParamConstants::RESPONSE_TEXT]) && count($headers[HttpParamConstants::RESPONSE_TEXT]) > 0) {
                $invocationContext->setResponseText(array_values($headers[HttpParamConstants::RESPONSE_TEXT])[0]);
            }

            if ($object !== null && $object->getInvocationContext() !== null) {
                $invocationContext->setSecret($object->getInvocationContext()->getSecret());
            }
        }

        return $invocationContext;
    }

    private static function wrapCustomersPayments(\Orbipay\PaymentsApi\Client\Swagger\Model\CustomersPaymentsResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $payments = new Payments();

        $payments->setUrl($response->getUrl());

        return $payments;
    }

    private static function wrapCustomersFundingAccounts(\Orbipay\PaymentsApi\Client\Swagger\Model\CustomersFundingAccountsResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $fundingAccounts = new FundingAccounts();

        $fundingAccounts->setUrl($response->getUrl());

        return $fundingAccounts;
    }

    private static function wrapCustomersPaymentSetups(\Orbipay\PaymentsApi\Client\Swagger\Model\CustomersPaymentSetupsResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $paymentSetups = new PaymentSetups();

        $paymentSetups->setUrl($response->getUrl());

        return $paymentSetups;
    }

    public static function wrapRefund(\Orbipay\PaymentsApi\Client\Swagger\Model\RefundResponse $response)
    {
        if ($response === null) {
            return null;
        }

        $refund = new Refund($response->getId());
        $refund->setUrl($response->getUrl());
        $refund->withDetails(RefundType::fromValue($response->getRefundType()), $response->getRefundAmount());
        $refund->setExpectedRefundSettlementDate($response->getExpectedRefundSettlementDate());
        $refund->withStatus(RefundStatus::fromValue($response->getStatus()));
        $refund->setConfirmationNumber($response->getConfirmationNumber());
        $refund->withReference($response->getRefundReference());
        $refund->setSourcePayment(self::wrapSourcePayment($response->getSourcePayment()));
        $refund->withCustomFields($response->getCustomFields());
        $refund->setFeeRefund(self::wrapFeeRefund($response->getFeeRefund()));
        $refund->setPaymentNetworkResponse(self::wrapPaymentNetworkResponse($response->getPaymentNetworkResponse()));
        $refund->setRefundDate($response->getRefundDate());
        $refund->setRefundEntryDate($response->getRefundEntryDate());
        $refund->setAuditInfo(self::wrapAuditInfo($response->getAuditInfo()));

        return $refund;
    }

    public static function wrapSourcePayment(\Orbipay\PaymentsApi\Client\Swagger\Model\SourcePayment $response = null)
    {
        if ($response === null) {
            return null;
        }

        $sourcePayment = new Payment($response->getId());
        $sourcePayment->setCustomer(self::wrapSourcePaymentCustomerResponse($response->getCustomer()));
        $sourcePayment->toAccount(self::wrapSourcePaymentCustomerAccountResponse($response->getCustomerAccount()));
        $sourcePayment->fromAccount(self::wrapFundingAccountSnapshotResponse($response->getFundingAccount()));
        $sourcePayment->setUrl($response->getUrl());

        return $sourcePayment;
    }

    public static function wrapCustomers(Customers $customers, CustomersResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        if (is_array($response->getList())) {
            $list = array();
            /** @var Customer $customerResponse */
            foreach ($response->getList() as $customerResponse) {
                $list[] = self::wrapCustomer($customerResponse);
            }
            $customers->setList($list);
        }
        $customers->setUrl($response->getUrl());
        $customers->setFromIndex($response->getFromIndex());
        $customers->setToIndex($response->getToIndex());
        $customers->setTotalResultsCount($response->getTotalResultsCount());
        $customers->setHasMoreResults($response->getHasMoreResults());
        if (is_string($response->getUrl())) {
            $customers->setQueryId(substr($response->getUrl(), strrpos($response->getUrl(), RestConstants::QUERY_ID_DELIMITER) + 1));
        }
        return $customers;
    }

    public static function wrapCustomerAccounts(CustomerAccounts $customerAccounts, CustomerAccountsResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        if (is_array($response->getList())) {
            $list = array();
            /** @var CustomerAccount $customerAccountResponse */
            foreach ($response->getList() as $customerAccountResponse) {
                $list[] = self::wrapCustomerAccount($customerAccountResponse);
            }
            $customerAccounts->setList($list);
        }
        $customerAccounts->setUrl($response->getUrl());
        $customerAccounts->setFromIndex($response->getFromIndex());
        $customerAccounts->setToIndex($response->getToIndex());
        $customerAccounts->setTotalResultsCount($response->getTotalResultsCount());
        $customerAccounts->setHasMoreResults($response->getHasMoreResults());
        if (is_string($response->getUrl())) {
            $customerAccounts->setQueryId(substr($response->getUrl(), strrpos($response->getUrl(), RestConstants::QUERY_ID_DELIMITER) + 1));
        }

        return $customerAccounts;
    }

    public static function wrapFundingAccounts(FundingAccounts $fundingAccounts, FundingAccountsResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        if (is_array($response->getList())) {
            $list = array();
            /** @var FundingAccount $fundingAccountResponse */
            foreach ($response->getList() as $fundingAccountResponse) {
                $list[] = self::wrapFundingAccount($fundingAccountResponse);
            }
            $fundingAccounts->setList($list);
        }
        $fundingAccounts->setUrl($response->getUrl());
        $fundingAccounts->setFromIndex($response->getFromIndex());
        $fundingAccounts->setToIndex($response->getToIndex());
        $fundingAccounts->setTotalResultsCount($response->getTotalResultsCount());
        $fundingAccounts->setHasMoreResults($response->getHasMoreResults());
        if (is_string($response->getUrl())) {
            $fundingAccounts->setQueryId(substr($response->getUrl(), strrpos($response->getUrl(), RestConstants::QUERY_ID_DELIMITER) + 1));
        }

        return $fundingAccounts;
    }

    public static function wrapRefunds(Refunds $refunds, RefundsResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        if (is_array($response->getList())) {
            $list = array();
            /** @var Refund $refundResponse */
            foreach ($response->getList() as $refundResponse) {
                $list[] = self::wrapRefund($refundResponse);
            }
            $refunds->setList($list);
        }
        $refunds->setUrl($response->getUrl());
        $refunds->setFromIndex($response->getFromIndex());
        $refunds->setToIndex($response->getToIndex());
        $refunds->setTotalResultsCount($response->getTotalResultsCount());
        $refunds->setHasMoreResults($response->getHasMoreResults());
        if (is_string($response->getUrl())) {
            $refunds->setQueryId(substr($response->getUrl(), strrpos($response->getUrl(), RestConstants::QUERY_ID_DELIMITER) + 1));
        }
        return $refunds;
    }

    public static function wrapPayments(Payments $payments, PaymentsResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        if (is_array($response->getList())) {
            $list = array();
            /** @var Payment $paymentResponse */
            foreach ($response->getList() as $paymentResponse) {
                $list[] = self::wrapPayment($paymentResponse);
            }
            $payments->setList($list);
        }
        $payments->setUrl($response->getUrl());
        $payments->setFromIndex($response->getFromIndex());
        $payments->setToIndex($response->getToIndex());
        $payments->setTotalResultsCount($response->getTotalResultsCount());
        $payments->setHasMoreResults($response->getHasMoreResults());
        if (is_string($response->getUrl())) {
            $payments->setQueryId(substr($response->getUrl(), strrpos($response->getUrl(), RestConstants::QUERY_ID_DELIMITER) + 1));
        }
        return $payments;
    }

    public static function wrapPaymentSetups(PaymentSetups $paymentSetups, PaymentSetupsResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        if (is_array($response->getList())) {
            $list = array();
            /** @var PaymentSetup $paymentSetupResponse */
            foreach ($response->getList() as $paymentSetupResponse) {
                $list[] = self::wrapPaymentSetup($paymentSetupResponse);
            }
            $paymentSetups->setList($list);
        }
        $paymentSetups->setUrl($response->getUrl());
        $paymentSetups->setFromIndex($response->getFromIndex());
        $paymentSetups->setToIndex($response->getToIndex());
        $paymentSetups->setTotalResultsCount($response->getTotalResultsCount());
        $paymentSetups->setHasMoreResults($response->getHasMoreResults());
        if (is_string($response->getUrl())) {
            $paymentSetups->setQueryId(substr($response->getUrl(), strrpos($response->getUrl(), RestConstants::QUERY_ID_DELIMITER) + 1));
        }

        return $paymentSetups;
    }

    public static function wrapErrors(ApiException $apiException)
    {
        /** @var \Orbipay\PaymentsApi\Client\Swagger\Model\ErrorResponseVO $response_object */
        $response_object = $apiException->getResponseObject();
        $getErrors = [];
        if ($response_object !== null && count($response_object->getErrors()) > 0) {
            for ($i = 0; $i < count($response_object->getErrors()); $i++) {
                $e = $response_object->getErrors()[$i];
                $getErrors[] = new Error($e->getCode(), $e->getMessage(), $e->getField());
            }
        }
        return $getErrors;
    }

    public static function wrapSimplePayment(\Orbipay\PaymentsApi\Client\Swagger\Model\SimplePaymentResponse $response = null)
    {
        if ($response === null) {
            return null;
        }

        $payment = new SimplePayment($response->getId());
        $payment->withDetails(PaymentAmountType::fromValue($response->getPaymentAmountType()), $response->getAmount(), $response->getPaymentDate(), $response->getPaymentRequestDate(), $response->getCardCvvNumber());
        $payment->withStatus(PaymentStatus::fromValue($response->getStatus()));
        $payment->withReference($response->getPaymentReference());
        $payment->withCustomFields($response->getCustomFields());
        $payment->fromAccount(self::wrapFundingAccount($response->getFundingAccount()));
        $payment->toAccount(self::wrapCustomerAccount($response->getCustomerAccount()));
        $payment->forCustomer(self::wrapCustomer($response->getCustomer()));
        $payment->setUrl($response->getUrl());
        $payment->setConfirmationNumber($response->getConfirmationNumber());
        $payment->setPaymentEntryDate($response->getPaymentEntryDate());
        $payment->setPaymentReturnDate($response->getPaymentReturnDate());
        $payment->setReturnCode($response->getReturnCode());
        $payment->setPaymentMethod(PaymentMethod::fromValue($response->getPaymentMethod()));
        $payment->setPaymentScheduleType(PaymentScheduleType::fromValue($response->getPaymentScheduleType()));
        $payment->setExpectedPaymentSettlementDate($response->getExpectedPaymentSettlementDate());
        $payment->setCurrencyCode3d($response->getCurrencyCode3d());
        $payment->setFee(self::wrapFee($response->getFee()));
        $payment->setAuditInfo(self::wrapAuditInfo($response->getAuditInfo()));
        $payment->setPaymentNetworkResponse(self::wrapPaymentNetworkResponse($response->getPaymentNetworkResponse()));

        return $payment;
    }

    private static function wrapAuditInfo(\Orbipay\PaymentsApi\Client\Swagger\Model\AuditInfo $response = null)
    {
        if ($response === null) {
            return null;
        }

        $auditInfo = new AuditInfo();

        $auditInfo->setCreated(self::wrapCreatedVO($response->getCreated()));
        $auditInfo->setLastModified(self::wrapLastModifiedVO($response->getLastModified()));

        return $auditInfo;
    }

    private static function wrapCreatedVO(\Orbipay\PaymentsApi\Client\Swagger\Model\ResourceAuditVO $response = null)
    {
        if ($response === null) {
            return null;
        }

        $created = new Created();

        $created->setChannel($response->getChannel());
        $created->setTimeStamp($response->getTimestamp());
        $created->setRequestor($response->getRequestor());
        $created->setRequestorType($response->getRequestorType());

        return $created;
    }

    private static function wrapLastModifiedVO(\Orbipay\PaymentsApi\Client\Swagger\Model\ResourceAuditVO $response = null)
    {
        if ($response === null) {
            return null;
        }

        $lastModified = new LastModified();

        $lastModified->setChannel($response->getChannel());
        $lastModified->setTimeStamp($response->getTimestamp());
        $lastModified->setRequestor($response->getRequestor());
        $lastModified->setRequestorType($response->getRequestorType());

        return $lastModified;
    }
}
